#ifndef CLAUSESDRIVER_H_
#define CLAUSESDRIVER_H_
 
#include <string>  
#include "y.tab.h" 
#include "Clause.h"
#include "ClausesInput.h"
#include "InputSet.h"
#include <set>

// Announce to Flex the prototype we want for lexing function, ... 
# define YY_DECL \
yy::ClausesParser::token_type \
yylex (yy::ClausesParser::semantic_type* yylval, \
yy::ClausesParser::location_type* yylloc, \
ClausesDriver& driver) 
// ... and declare it for the parsers sake. 
YY_DECL; 

class ClausesDriver : public ClausesInput
{
        std::string filename;
public:
	ClausesDriver(const std::string& f);
	virtual ~ClausesDriver();
	
        void execute();
        InputSet *getInputSet();

	NS_PROOFLIB::ProofDatabase *pdb;
	VariableNameGenerator *vng;
	std::set<Clause*> *result;
	// Handling the scanner. 
	void scan_begin (); 
	void scan_end (); 
	bool trace_scanning; 
	
	// Handling the parser. 
	void parse (const std::string& f); 
	std::string file; 
	bool trace_parsing; 

	// Error handling. 
	void error (const yy::location& l, const std::string& m); 
	void error (const std::string& m); 

};

#endif /*CLAUSESDRIVER_H_*/
