#include "BinaryDeductionEngine.h"

#include <iostream>

using namespace std;

BinaryDeductionEngine::BinaryDeductionEngine()
{
}

BinaryDeductionEngine::~BinaryDeductionEngine()
{
}

Resolvent* BinaryDeductionEngine::deduce(Clause *target)
{
	// start measuring time
	time_t seconds;
	seconds = time (NULL);

	ResolvingSequence *rseq;
	Resolvent *resolvent;
	bool validResolvent;
	bool proofCompleted = false;
	Clause *c1, *c2;
	ClauseStrategy *strategy;
	while (!proofCompleted && !env->getRefinement()->isEmpty())
	{
		// run expires
		if (env->getTimeout() > 0 && time(NULL) - seconds > env->getTimeout())
		{
			throw new ATPException("Timeout reached");
		}

		// refinement may return NULL, then just loop again

		rseq = env->getRefinement()->getNextResolvingSequence();
		if (rseq == NULL)
		{
			break;
		}
		strategy = rseq->getClauseStrategy();
		validResolvent = true;
		if (rseq->getSize() < 2)
		{
			throw new RefinementException("Resolving sequence size is smaller than 2");
		}
		//TODO to remove when support for longer sequences is implemented
		if (rseq->getSize() > 2)
		{
			throw new ATPException("Right now only sequences with size = 2 are supported");
		}
		c1 = rseq->getNextClause();
		c2 = rseq->getNextClause();

		// BFS
		list<Resolvent *> resolvents = executeBFS(env, c1, c2, strategy);
		// iterates over all resolvents and attempt each with remaining of resolving sequence.
		//TODO - right now only seq of length 2 are supported. add support for more.

		// if no resolvents then sequence should be terminated
		if (resolvents.empty())
		{
			//TODO when adding support for longer sequences
		}
		else
		{
			for (list<Resolvent *>::const_iterator it = resolvents.begin(); it != resolvents.end(); ++it)
			{
				resolvent = *it;
				if (isTargetResolvent(resolvent, target))
				{
					proofCompleted = true;
					break;
				}

				// postprocess new resolvent
				postProcess(resolvent);

			}
		}
	}

	if (proofCompleted)
	{
		return resolvent;
	}
	else
	{
		return NULL;
	}
}

list<Resolvent *> BinaryDeductionEngine::executeBFS(ProofEnvironment *env, Clause *cc1, Clause *cc2, ClauseStrategy *cs)
{
	list<Resolvent *> ret;
	Clause *c1 = cc1;
	Clause *c2 = cc2;

	ClauseStrategyRuling *csr;
	set<Resolvent *, ClausePointerCmp> resolvents;
	Resolvent *resolvent;

	typedef std::set<Resolvent*,ClausePointerCmp>::const_iterator LCI;

	for (int i = 0; i<c1->getPositiveLiterals().size(); i++)
    {
		for (int j = 0; j<c2->getPositiveLiterals().size(); j++)
		{
			csr = cs->getRuling(c1, c2, i, j, true, true);
			// Rulling is valid
			if (csr != NULL)
			{
				csr->addParent(c1);
				csr->addParent(c2);
				resolvents = env->getResolvingUnit()->resolve(c1, c2, csr);
				for (LCI it = resolvents.begin(); it != resolvents.end(); ++it)
				{
					resolvent = *it;
					if (resolvent->getType() == Resolvent::NORMAL || resolvent->getType() == Resolvent::EMPTY)
					{
						ret.push_back(resolvent);
					}
				}
			}
		}

		for (int j = 0; j<c2->getNegativeLiterals().size(); j++)
		{
			csr = cs->getRuling(c1, c2, i, j, true, false);
			// Rulling is valid
			if (csr != NULL)
			{
				csr->addParent(c1);
				csr->addParent(c2);
				resolvents = env->getResolvingUnit()->resolve(c1, c2, csr);
				for (LCI it = resolvents.begin(); it != resolvents.end(); ++it)
				{
					resolvent = *it;
					if (resolvent->getType() == Resolvent::NORMAL || resolvent->getType() == Resolvent::EMPTY)
					{
						ret.push_back(resolvent);
					}
				}
			}
		}
	}

	for (int i = 0; i<c1->getNegativeLiterals().size(); i++)
    {

		for (int j = 0; j<c2->getPositiveLiterals().size(); j++)
		{
			csr = cs->getRuling(c1, c2, i, j, false, true);
			// Rulling is valid
			if (csr != NULL)
			{
				csr->addParent(c1);
				csr->addParent(c2);
				resolvents = env->getResolvingUnit()->resolve(c1, c2, csr);
				for (LCI it = resolvents.begin(); it != resolvents.end(); ++it)
				{
					resolvent = *it;
					if (resolvent->getType() == Resolvent::NORMAL || resolvent->getType() == Resolvent::EMPTY)
					{
						ret.push_back(resolvent);
					}
				}
			}
		}

		for (int j = 0; j<c2->getNegativeLiterals().size(); j++)
		{
			csr = cs->getRuling(c1, c2, i, j, false, false);
			// Rulling is valid
			if (csr != NULL)
			{
				csr->addParent(c1);
				csr->addParent(c2);
				resolvents = env->getResolvingUnit()->resolve(c1, c2, csr);
				for (LCI it = resolvents.begin(); it != resolvents.end(); ++it)
				{
					resolvent = *it;
					if (resolvent->getType() == Resolvent::NORMAL || resolvent->getType() == Resolvent::EMPTY)
					{
						ret.push_back(resolvent);
					}
				}
			}
		}
	}

	return ret;
}

void BinaryDeductionEngine::postProcess(Resolvent *r)
{
	typedef std::set<ResolventPostProcessor*, PostProcessorPriorityCmp>::const_iterator LCI;
	for (LCI i = env->getPostProcessors().begin(); i != env->getPostProcessors().end(); ++i)
	{
			ResolventPostProcessor *rpp = *i;
			// in case the processor order to stop processing
			// this resolvent
			if (!rpp->postProcess(r))
			{
				break;
			}
	}
}

bool BinaryDeductionEngine::isTargetResolvent(Resolvent *r, Clause *c)
{
	return c == NULL && r->getType() == Resolvent::EMPTY;
}
