#include "CEResXMLNodePrint.h"

using namespace std;

CEResXMLNodePrint::CEResXMLNodePrint(ProofEnvironment *e)
{
	env = e;
    ofs.open(env->getOutputFilename().c_str());
}

CEResXMLNodePrint::~CEResXMLNodePrint()
{

}

void CEResXMLNodePrint::initialize()
{
    ofs<<"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<!DOCTYPE proofdatabase\nSYSTEM \"http://www.logic.at/ceres/xml/4.0/proofdatabase.dtd\">\n<proofdatabase>\n\t<definitionlist/>\n\t<axiomset/>\n";
}

void CEResXMLNodePrint::finalize()
{
    ofs<<"\t\t<variabledefinitions/>\n</proofdatabase>\n";
    ofs.close();
}

void CEResXMLNodePrint::printNode(Factor& node)
{
    if (node.isPositive())
    {
        ofs<<"\t\t<rule type=\"factr\" symbol=\"f:r\">\n";
    }
    else
    {
        ofs<<"\t\t<rule type=\"factl\" symbol=\"f:l\">\n";
    }
    printSequent(node);
}

void CEResXMLNodePrint::printNode(Clause& node)
{
	// first check it has no variants, otherwise it should be printed by printNode(Variant)
	Variant *v = getVariant(node);
	if (v == NULL)
	{
	    ofs<<"\t\t<rule type=\"init\" symbol=\"\">\n";
	    printSequent(node);
	    cout<<node.toString()<<endl;
	    cout<<"--------------------------"<<endl;
	    cout<<node.getClauseContext()->toStringOnNewLines()<<endl<<endl;
	    ofs<<"\t\t</rule>\n";
	}
	else
	{
		ofs<<"\t\t<rule type=\"init\" symbol=\"\">\n";
	    printSequent(*v);
	    cout<<v->toString()<<endl;
	    cout<<"--------------------------"<<endl;
	    cout<<v->getClauseContext()->toStringOnNewLines()<<endl<<endl;
	    ofs<<"\t\t</rule>\n";
	}
}

Variant *CEResXMLNodePrint::getVariant(ProofNode& p)
{
	vector<ProofNode*> *children = p.getChildren();
	for (vector<ProofNode*>::const_iterator it = children->begin(); it != children->end(); ++it)
	{
		if (typeid(**it) == typeid(Variant))
		{
			return dynamic_cast<Variant*>(*it);
		}
	}
	return NULL;
}

void CEResXMLNodePrint::printNode(Unifier& node)
{
}

void CEResXMLNodePrint::printNode(Variant& node)
{
	//is being printed in clause and resolvent
}

void CEResXMLNodePrint::printNode(Resolvent& node)
{
    ofs<<"\t\t<rule type=\"res\" symbol=\"r\">\n";
	// first check it has no variants, otherwise it should be printed by printNode(Variant)
	Variant *v = getVariant(node);
	if (v == NULL)
	{
	    printSequent(node);
	    cout<<node.toString()<<endl;
	    cout<<"--------------------------"<<endl;
	    cout<<node.getClauseContext()->toStringOnNewLines()<<endl<<endl;
	}
	else
	{
	    printSequent(*v);
	    cout<<v->toString()<<endl;
	    cout<<"--------------------------"<<endl;
	    cout<<v->getClauseContext()->toStringOnNewLines()<<endl<<endl;
	}
}

void CEResXMLNodePrint::printNode(ParaModulant& node)
{
	char *v;
	if (node.isPositive())
	{
		v = "p:r_1";
	}
	else
	{
		v = "p:l_1";
	}
    ofs<<"\t\t<rule type=\"parar1\" symbol=\""<<v<<"\" param=\"(1)\">\n";
    printSequent(node);
}

void CEResXMLNodePrint::printSequent(Clause &c)
{
    ofs<<"\t\t\t<sequent>\n";
    ofs<<"\t\t\t\t<formulalist>\n";

    vector<NS_PROOFLIB::Term*> neg = c.getNegativeLiterals();
    typedef vector<NS_PROOFLIB::Term*>::const_iterator VIT;

    for (VIT it = neg.begin(); it != neg.end(); ++it)
    {
        printFormula(*it, c.getClauseContext());
    }

    ofs<<"\t\t\t\t</formulalist>\n";

    ofs<<"\t\t\t\t<formulalist>\n";

    vector<NS_PROOFLIB::Term*> pos = c.getPositiveLiterals();

    for (VIT it = pos.begin(); it != pos.end(); ++it)
    {
        printFormula(*it, c.getClauseContext());
    }

    ofs<<"\t\t\t\t</formulalist>\n";
    ofs<<"\t\t\t</sequent>\n";
}

void CEResXMLNodePrint::printFormula(NS_PROOFLIB::Term *tt, ClauseContext *cc)
{
    NS_PROOFLIB::Term *t = cc->getVariableInContext(tt);
    ofs<<"\t\t\t\t\t<constantatomformula symbol=\"" << t->getSymbol() <<"\">\n";
    if (typeid(*t) == typeid(NS_PROOFLIB::Function))
	{
        NS_PROOFLIB::Function *f = dynamic_cast<NS_PROOFLIB::Function *>(t);
        for (unsigned int i=1; i<=f->getArity(); i++)
        {
            printTerm(f->getSub(i), cc);
        }
	}
	ofs<<"\t\t\t\t\t</constantatomformula>";
}

void CEResXMLNodePrint::printTerm(NS_PROOFLIB::Term *tt, ClauseContext *cc)
{
    NS_PROOFLIB::Term *t = cc->getVariableInContext(tt);
    if (typeid(*t) == typeid(NS_PROOFLIB::Function))
	{
        NS_PROOFLIB::Function *f = dynamic_cast<NS_PROOFLIB::Function *>(t);
        ofs<<"\t\t\t\t\t\t<function symbol=\""<< t->getSymbol() <<"\">\n";
        for (unsigned int i=1; i<=f->getArity(); i++)
        {
            printTerm(f->getSub(i), cc);
        }
        ofs<<"\t\t\t\t\t\t</function>\n";
	}
	else if (typeid(*t) == typeid(NS_PROOFLIB::Constant))
	{
	    ofs<<"\t\t\t\t\t\t<constant symbol=\""<< t->getSymbol() <<"\"/>\n";
    }
    else if (typeid(*t) == typeid(NS_PROOFLIB::Variable))
	{
	    ofs<<"\t\t\t\t\t\t<variable symbol=\""<< t->getSymbol() <<"\"/>\n";
    }
}

void CEResXMLNodePrint::beginNode(ProofNode &node)
{

}

void CEResXMLNodePrint::endNode(ProofNode &node)
{
    if ( (typeid(node) == typeid(Resolvent)) || (typeid(node) == typeid(Factor))  || (typeid(node) == typeid(ParaModulant)) )
        ofs<<"\t\t</rule>\n";
}

void CEResXMLNodePrint::beginChildren(ProofNode& node)
{

}

void CEResXMLNodePrint::endChildren(ProofNode &node)
{

}

void CEResXMLNodePrint::beginTree()
{
    ofs<<"\t<proof symbol=\"refutation\" calculus=\"RES\">\n";
}

void CEResXMLNodePrint::endTree()
{
    ofs<<"\t</proof>\n";
}
