#include "Clause.h"
#include "myassert.h"

using namespace std;
using namespace NS_PROOFLIB;

Clause::Clause(vector<Term *> p, vector<Term *> n, VariableNameGenerator *v) : DataProofNode()
{
    pos = p;
    neg = n;
    vng = v;
    info = new ClauseInfo();
}

Clause::~Clause()
{
    // delete all terms only at the end as they are mixed between clauses
    // delete both the terms and the variable name generator in the main at the end
    /*for (vector<Term *>::const_iterator it = pos.begin(); it != pos.end(); ++it)
    {
        (*it)->detach();
    }
    for (vector<Term *>::const_iterator it = neg.begin(); it != neg.end(); ++it)
    {
        (*it)->detach();
    }*/
    delete context;
    delete info;
}

Clause *Clause::getVariant()
{
	map<Term*, Term*> variants;
	vector<Term*> newPos;
	vector<Term*> newNeg;

	// iterate over the variables and change them, also change the variables in the context
	typedef std::vector<Term*>::const_iterator LI;
	for (LI it = pos.begin(); it != pos.end(); ++it)
	{
		newPos.push_back(mapVariants((*it), variants));
	}
	for (LI it = neg.begin(); it != neg.end(); ++it)
	{
		newNeg.push_back(mapVariants((*it), variants));
	}

	// change context variables
	ClauseContext *cc = new ClauseContext();
	typedef map<Variable*, Term*>::const_iterator MI;
	for (MI it = context->contextMap.begin(); it != context->contextMap.end(); ++it)
	{
		pair<Variable*, Term*> p = *it;
		//TODO check why elements in context get nulled! check on input - {{r(f(X))}, {p(X)}} {{p(f(X)), r(X)}, {}} {{},{r(X)}}
		if (p.first != NULL && p.second != NULL)
		{
			Term *vt = mapVariants(p.first, variants);
			Variable *v = dynamic_cast<Variable*>(vt);
			cc->mapVariableToTerm(v, mapVariants(p.second, variants));
		}
	}

	Clause *ret = new Variant(newPos, newNeg, vng);
	ret->setClauseContext(cc);
	ret->addParent(this);

	return ret;
}

Term *Clause::mapVariants(Term *t, map<Term*, Term*> &variants)
{
	ASSERT(t != NULL);
	// variables and functions should be modified
	Term *tt = variants[t];
	if (tt == NULL)
	{
		// constants should be return as is
		if (typeid(*t) == typeid(Constant))
		{
			tt = t;
		}
		// variabes should be renamed
		else if (typeid(*t) == typeid(Variable))
		{
			Variable *v = dynamic_cast<Variable*>(t);
			tt = vng->generateNameFor(v);
		}
		// functions should be parsed and the be called recursively
		else
		{
			Function *f = dynamic_cast<Function*>(t);
			TermList l;

			for (unsigned int i=1; i<=f->getArity(); i++)
			{
				l.addRight(mapVariants(f->getSub(i), variants));
			}
			tt = vng->pdb->newFunction(f->getSymbol().c_str(), l);
		}
		variants[t] = tt;
	}
	return tt;
}

VariableNameGenerator *Clause::getVariableNameGenerator()
{
	return vng;
}

vector<NS_PROOFLIB::Term*> &Clause::getPositiveLiterals()
{
    return pos;
}

vector<NS_PROOFLIB::Term*> &Clause::getNegativeLiterals()
{
    return neg;
}

ClauseContext *Clause::getClauseContext()
{
	return context;
}

void Clause::setClauseContext(ClauseContext *cc)
{
	context = cc;
}

string Clause::toString()
{
	string ret;
	Term *t;
	typedef std::vector<Term*>::const_iterator LI;
    for (LI it = neg.begin(); it != neg.end(); )

	{
		t = *it;
		ret.append(termToString(t));
		if ((++it) != neg.end()) ret.append(", ");
	}
	ret.append(" :- ");
	for (LI it = pos.begin(); it != pos.end(); )
	{
		t = *it;
		ret.append(termToString(t));
		if ((++it) != pos.end()) ret.append(", ");
	}
	//ret.append(context->toString());
	return ret;
}

bool Clause::operator<(const Clause &t) const
{
	if (pos.size() + neg.size() < t.pos.size() + t.neg.size()) return true;
	else if (pos.size() + neg.size() > t.pos.size() + t.neg.size()) return false;
	else if(pos.size() < t.pos.size()) return true;
	else if(pos.size() > t.pos.size()) return false;
	for (unsigned int i=0; i<pos.size(); i++)
	{
		//TODO add operator < to terms
		// for now return the one with bigger address
		if (*(pos.at(i)) != *(t.pos.at(i))) return this < &t;
	}
	for (unsigned int i=0; i<neg.size(); i++)
	{
		if (*(neg.at(i)) != *(t.neg.at(i))) return this < &t;
	}
	if (*context != *t.context) return false;
	return true;
}

set<Variable*> Clause::getAllVariables()
{
	set<Variable*> ret;
	typedef vector<Term*>::const_iterator IT;

	if (!pos.empty())
	{
		for (IT it = pos.begin(); it != pos.end(); ++it)
		{
			addVariables(*it,ret);
		}
	}
	if (!neg.empty())
	{
		for (IT it = neg.begin(); it != neg.end(); ++it)
		{
			addVariables(*it,ret);
		}
	}

	set<Variable*> keys = context->getKeys();
	ret.insert(keys.begin(), keys.end());
	return ret;
}

void Clause::addVariables(Term *told, set<Variable*> &st) const
{
	// add the term if it is a variable before being mapped in context
	if (typeid(*told) == typeid(Variable))
	{
		st.insert(dynamic_cast<Variable*>(told));
	}

	Term *t = context->getVariableInContext(told);
	if (typeid(*t) == typeid(Variable))
	{
		st.insert(dynamic_cast<Variable*>(t));
	}
	else if (typeid(*t) == typeid(Function))
	{
		Function *f= dynamic_cast<Function *>(t);

		for (unsigned int i=1; i<=f->getArity(); i++)
		{
			addVariables(f->getSub(i), st);
		}
	}
}

bool ClausePointerCmp::operator()(const Clause* c1, const Clause* c2) const
{
	return *c1 < *c2;
}

string Clause::termToString(Term* t)
{
    // changing = to infix notation
    if (t->getSymbol() == "=" && (typeid(*t) == typeid(Function)))
    {
        Function *f = dynamic_cast<Function *>(t);
        if (f->getArity() != 2)
        {
            throw new ATPException("The predicate = must contains two terms only!");
        }
        string ret;
        ret.append(context->getStringValueInContext(f->getSub(1)));
        ret.append(" = ");
        ret.append(context->getStringValueInContext(f->getSub(2)));
        return ret;
    }
    else
    {
        return context->getStringValueInContext(t);
    }
    // the recursive descent is being done in the context
	/*if (typeid(*t) == typeid(Variable))
	{
		return context->getStringValueInContext(t);
	}
	else if (typeid(*t) == typeid(Function))
	{
		Function *f= dynamic_cast<Function *>(t);
		string ret = f->getSymbol();
		ret.append("(");
		for (unsigned int i=1; i<=f->getArity(); i++)
		{
			if (i > 1)
			{
				ret.append(",");
			}
			ret.append(termToString(f->getSub(i)));
		}
		ret.append(")");
		return ret;
	}
	else
	{
		return t->getStringRep();
	}*/
}

ClauseInfo *Clause::getClauseInfo()
{
	return info;
}

Clause *Clause::getGroundInstance()
{
	vector<Term*> newPos(pos.begin(), pos.end());
	vector<Term*> newNeg(neg.begin(), neg.end());

	ClauseContext *cc = new ClauseContext();
	cc->contextMap.insert(context->contextMap.begin(), context->contextMap.end());

	// iterate over the terms and when getting to a variable
	// bind it to a new constant
	int indices = 0;
	for (vector<Term*>::const_iterator it = pos.begin(); it != pos.end(); ++it)
	{
		recGround(*it, cc, indices);
	}
	for (vector<Term*>::const_iterator it = neg.begin(); it != neg.end(); ++it)
	{
		recGround(*it, cc, indices);
	}
	Clause *ret = new Clause(newPos, newNeg, vng);
	ret->setClauseContext(cc);

	return ret;
}

void Clause::recGround(NS_PROOFLIB::Term* tt, ClauseContext* cc, int& indices)
{
	Term *t = cc->getVariableInContext(tt);

	// t1 is a variable
	if (typeid(*t) == typeid(Variable))
	{
		Variable *v = dynamic_cast<Variable*>(t);
		ostringstream oss;
		oss << "TMP_CONSTANT_" <<indices;
		string s(oss.str());
		indices++;
		// TODO delete these constants when the ground clause is deleted!!!
		Constant *c = vng->pdb->newConstant(s.c_str());
		cc->mapVariableToTerm(v, c);
	}
	else if (typeid(*t) == typeid(Function))
	{
		Function *f = dynamic_cast<Function *>(t);
		for (unsigned int i=1; i<=f->getArity(); i++)
		{
			recGround(f->getSub(i), cc, indices);
		}
	}
}
/*
void Clause::clean()
{
	// get variables in context
	set<Variable*> inContext = context->getKeys();
	// get variables in clause
	set<Variable*> variables = getVariables();
	// for each variable in context, if not in clause, delete from context
}

set<Variable*> Clause::getVariables()
{
	set<Variable*> variables = getVariables();
	for (vector<Term*>::const_iterator it = pos.begin(); it != pos.end(); ++it)
	{
		addVariables(variables, *it);
	}
	for (vector<Term*>::const_iterator it = neg.begin(); it != neg.end(); ++it)
	{
		addVariables(variables, *it);
	}
}

void Clause::addVariables(set<Variable*> &variables, Term *t)
{
	if (typeid(*t) == typeid(Variable))
	{
		variables.insert(dynamic_cast<Variable*>(t));
	}
	else if (typeid(*t) == typeid(Function))
	{
		Function *f = dynamic_cast<Function *>(t);
		for (unsigned int i=1; i<=f->getArity(); i++)
		{
			addVariables(variables, f->getSub(i));
		}
	}
}*/

void Variant::print(NodePrint& printer)
{
    printer.beginNode(*this);
    printer.printNode(*this);
    printer.beginChildren(*this);
    for (vector<ProofNode*>::const_iterator it = parents->begin(); it != parents->end();++it)
    {
        (*it)->print(printer);
    }
    printer.endChildren(*this);
    printer.endNode(*this);
}

void Clause::print(NodePrint& printer)
{
    printer.beginNode(*this);
    printer.printNode(*this);
    printer.beginChildren(*this);
    for (vector<ProofNode*>::const_iterator it = parents->begin(); it != parents->end();++it)
    {
        (*it)->print(printer);
    }
    printer.endChildren(*this);
    printer.endNode(*this);
}

Variant::Variant(vector<Term *> p, vector<Term *> n, VariableNameGenerator *v) : Clause(p,n,v)
{
}

Variant::~Variant()
{
}

void Factor::print(NodePrint& printer)
{
    printer.beginNode(*this);
    printer.printNode(*this);
    printer.beginChildren(*this);
    for (vector<ProofNode*>::const_iterator it = parents->begin(); it != parents->end();++it)
    {
        (*it)->print(printer);
    }
    printer.endChildren(*this);
    printer.endNode(*this);
}

Factor::Factor(bool isP, vector<Term *> p, vector<Term *> n, VariableNameGenerator *v) : Clause(p,n,v)
{
	isPos = isP;
}

bool Factor::isPositive()
{
    return isPos;
}

Factor::~Factor()
{
}

bool ClausePairCmp::operator()(const std::pair<Clause*,unsigned int> c1, const std::pair<Clause*,unsigned int> c2) const
{
	return *(c1.first) < *(c2.first);
}
