#include "ClauseContext.h"
#include <typeinfo>

using namespace NS_PROOFLIB;
using namespace std;

ClauseContext::ClauseContext()
{
}

ClauseContext::~ClauseContext()
{
}

void ClauseContext::clear()
{
	contextMap.clear();
}

void ClauseContext::mapVariableToTerm(Variable *t1, Term *t2)
{
	contextMap[t1] = t2;
}

string ClauseContext::getStringValueInContext(Term *t)
{
	if (typeid(*t) == typeid(Variable))
	{
		Variable *v = dynamic_cast<Variable*>(t);
		if (contextMap.count(v) > 0)
		{
			return getStringValueInContext(contextMap[v]);
		}
		else
		{
			return t->getStringRep();
		}
	}
	else if (typeid(*t) == typeid(Constant))
	{
		return t->getStringRep();
	}
	// function
	else
	{
		Function *f = dynamic_cast<Function *>(t);
		string ret(f->getSymbol());
		ret.append("(");
		for (unsigned int i=1; i<=f->getArity(); i++)
		{
			if (i > 1)
			{
				ret.append(",");
			}
			ret.append(getStringValueInContext(f->getSub(i)));
		}
		ret.append(")");
		return ret;
	}
}

Term *ClauseContext::getVariableInContext(Term *t)
{
	Term* ret;
	if (typeid(*t) == typeid(Variable))
	{
		Variable *v = dynamic_cast<Variable*>(t);
		if (contextMap.count(v) > 0)
		{
			return getVariableInContext(contextMap[v]);
		}
		else
		{
			ret = t;
		}
	}
	else
	{
		ret = t;
	}
	return ret;
}

string ClauseContext::toString()
{
	string ret = "{";
	typedef map<Variable*, Term*>::const_iterator MI;
	int i = 0;
	for (MI it = contextMap.begin(); it != contextMap.end(); ++it)
	{
		pair<Variable*, Term*> p = *it;
		if (i != 0)
		{
			ret.append(", ");
		}
		ret.append((p.first)->getStringRep());
		ret.append(" -> ");
		ret.append((p.second)->getStringRep());
		i++;
	}
	ret.append("}");
	return ret;
}

string ClauseContext::toStringOnNewLines()
{
	string ret;
	typedef map<Variable*, Term*>::const_iterator MI;
	int i = 0;
	for (MI it = contextMap.begin(); it != contextMap.end(); ++it)
	{
		pair<Variable*, Term*> p = *it;
		if (i != 0)
		{
			ret.append("\n");
		}
		ret.append((p.first)->getStringRep());
		ret.append(" -> ");
		ret.append((p.second)->getStringRep());
		i++;
	}
	return ret;
}

bool ClauseContext::operator==(ClauseContext &t) const
{
	typedef map<Variable*, Term*>::const_iterator MI;
	for (MI it = contextMap.begin(); it != contextMap.end(); ++it)
	{
		pair<Variable*, Term*> p = *it;
		if (t.contextMap.count(p.first) == 0) return false;
		else if (*(p.second) != *(t.contextMap[p.first])) return false;
	}
	return true;
}

bool ClauseContext::operator!=(ClauseContext &t) const
{
	return !(*this == t);
}

set<Variable*> ClauseContext::getKeys()
{
	set<Variable*> ret;
	for (map<Variable*, Term*>::const_iterator it = contextMap.begin(); it != contextMap.end(); ++it)
	{
		ret.insert((*it).first);
	}
	return ret;
}

const map<Variable*, Term*> &ClauseContext::getContextMap() const
{
    return contextMap;
}
