#ifndef CLAUSECONTEXT_H_
#define CLAUSECONTEXT_H_

#include "Variable.h"
#include "Constant.h"
#include "Term.h"
#include "Function.h"

#include <map>

class ClauseContext
{
	friend class Unifier;
	friend class Clause;
private:
	std::map<NS_PROOFLIB::Variable*, NS_PROOFLIB::Term*> contextMap;
public:
	ClauseContext();
	virtual ~ClauseContext();
	void clear();
	void mapVariableToTerm(NS_PROOFLIB::Variable *t1, NS_PROOFLIB::Term *t2);
	/*
	 * return the binding of a variable only, other terms are not changed
	 * also gets the last binding for a variable chain i.e. X->Y>Z->f(a) we get X->f(a)
	 * but for X->Y->f(Z) and Z->a we get X->f(Z) (and not X->f(a) as before)
	 */
	NS_PROOFLIB::Term *getVariableInContext(NS_PROOFLIB::Term *t);
	/*
	 * similar to the above but this time returns a string and not a variable
	 * and also will map also all variables inside the term so the above
	 * mismatch will not occur
	 */
	std::string getStringValueInContext(NS_PROOFLIB::Term *t);
	virtual std::string toString();
	virtual std::string toStringOnNewLines();
	bool operator==(ClauseContext &t) const;
	bool operator!=(ClauseContext &t) const;
	std::set<NS_PROOFLIB::Variable*> getKeys();
        const std::map<NS_PROOFLIB::Variable*, NS_PROOFLIB::Term*> &getContextMap() const;
};

#endif /*CLAUSECONTEXT_H_*/
