#include "InteractiveRefinement.h"

using namespace std;

InteractiveRefinement::InteractiveRefinement(ProofEnvironment *e)
{
	env = e;
	autoSteps = 0;
}

InteractiveRefinement::~InteractiveRefinement()
{
    delete env->getManagedRefinement();
}

void InteractiveRefinement::setUserCommand(UserCommand *uc)
{
	ucom = uc;
}

void InteractiveRefinement::initialize(InputSet *cs)
{
	env->getManagedRefinement()->initialize(cs);
	is = cs;
}

void InteractiveRefinement::finalize()
{
	env->getManagedRefinement()->finalize();
}

bool InteractiveRefinement::isEmpty() const
{
	return env->getManagedRefinement()->isEmpty();
}

ResolvingSequence *InteractiveRefinement::getNextResolvingSequence()
{
	if (autoSteps > 0)
	{
		autoSteps--;
		return env->getManagedRefinement()->getNextResolvingSequence();
	}
	else
	{
		UserReply *reply;
		try
		{
			// do not calculate the time the user choose
			time_t user_seconds;
			user_seconds = time (NULL);
			ResolvingSequence *ret;
			reply = ucom->getReplyFromUser(is, env->getManagedRefinement()->getIndicesMap(), env->getManagedRefinement()->getIndicesMessage());
			userTime += time(NULL) - user_seconds;
			vector<int> indices = reply->getIndices();

			switch (reply->getType())
			{
			case UserReply::AUTO:
				// automatic mode for x steps
				if (indices.size() == 1)
				{
					autoSteps = indices[0];
					ret = getNextResolvingSequence();
				}
				else
				{
					// error illegal argument in indices
				}
				break;

			case UserReply::INTERACTIVE:
				if (indices.size() == 2)
				{
					ret = env->getManagedRefinement()->getResolvingSequenceFromIndices(indices);
				}
				else
				{
					// error illegal argument in indices
				}

				break;

			case UserReply::MACRO:

				break;
			}

			return ret;
		}
		catch (RefinementException *re)
		{
			ucom->errorEncountered(re->getMessage());
		}
		catch (...)
		{
			delete reply;
		}
		return getNextResolvingSequence();
	}
}

void InteractiveRefinement::setEventPerformed(InputSetEvent ise)
{
	ucom->clausesEventPerformed(ise);
	env->getManagedRefinement()->setEventPerformed(ise);
}
