#include "LatexNodePrint.h"

using namespace std;

LatexNodePrint::LatexNodePrint(ProofEnvironment *e)
{
	env = e;
    ofs.open(env->getOutputFilename().c_str());
}

LatexNodePrint::~LatexNodePrint()
{

}

void LatexNodePrint::initialize()
{
    ofs<<"\\documentclass[11pt]{amsart}\n\\usepackage{qtree,amsmath}\n\\begin{document}\n\\tiny\n";
}

void LatexNodePrint::finalize()
{
    ofs<<"\n\\end{document}";
    ofs.close();
}

void LatexNodePrint::printNode(Factor& node)
{
    if (node.hasParents()) ofs<<".";
    ofs<<"{"<<node.toString()<<"}";
}

void LatexNodePrint::printNode(Clause& node)
{
    if (node.hasParents()) ofs<<".";
    ofs<<"{"<<node.toString()<<"}";
}
void LatexNodePrint::printNode(Unifier& node)
{
    ofs<<" .{";

    typedef map<NS_PROOFLIB::Variable*, NS_PROOFLIB::Term*>::const_iterator MI;
    const map<NS_PROOFLIB::Variable*, NS_PROOFLIB::Term*> contextMap = node.getContextMap();

    for (MI it = contextMap.begin(); it != contextMap.end(); )
    {
        pair<NS_PROOFLIB::Variable*, NS_PROOFLIB::Term*> p = *it;
        ofs<<(p.first)->getStringRep()<<" $\\mapsto$ "<<(p.second)->getStringRep();
        if (++it != contextMap.end()) ofs<<", ";
    }
    ofs<<"}";
}

void LatexNodePrint::printNode(Variant& node)
{
    if (node.hasParents()) ofs<<".";
    ofs<<"{"<<node.toString()<<"}";
}

void LatexNodePrint::printNode(Resolvent& node)
{
    if (node.hasParents()) ofs<<".";
    ofs<<"{"<<node.toString()<<"}";

}

void LatexNodePrint::printNode(ParaModulant& node)
{
    if (node.hasParents()) ofs<<".";
    ofs<<"{"<<node.toString()<<"}";

}

void LatexNodePrint::beginNode(ProofNode &node)
{
    ofs<<" [";
}

void LatexNodePrint::endNode(ProofNode &node)
{
    ofs<<" ] ";
}

void LatexNodePrint::beginChildren(ProofNode& node)
{

}

void LatexNodePrint::endChildren(ProofNode &node)
{

}

void LatexNodePrint::beginTree()
{
    ofs<<"\\Tree ";
}

void LatexNodePrint::endTree()
{
    ofs<<"\\\\\\\\\\\\";
}
