#ifndef POSITIVEREFINEMENT_H_
#define POSITIVEREFINEMENT_H_

#include "ManagedRefinement.h"

#include <vector>
#include <set>

/**
 * simple implementation keeping a boolean array to note positive elements
 * and the current two indices and then it compares every two elements in the vector
 * in order so the addition of new clauses will not impair the order (Although deletion will)
 * i.e. given the four elements A B C D we compare: A-B, A-C, B-C, A-D, B-D, C-D.
 * The addition on an element in the middle of the process will not change the ordering
 * i.e. having completed until A-D and then adding E, we do B-D and then D-E etc.
 * for the case interative refinement, we store in a set all the pairs that were
 * already done.
 * as equalities are considered as positive clasues (if there is an equality then there
 * should not be any negative literal as we we dont not compare two non positive clauses
 * and so no para modulation will happen) we must also compare all positive with one another
 */
class PositiveRefinement : public ManagedRefinement
{
	std::vector<Clause*> all;
	std::vector<bool> pos;
	unsigned int firstInd, secondInd;
	std::set<int> history; //contains a unique has of each pair, TODO make it commutative, assume maximal size
	static const int MAX_SIZE = 999999999;
	bool useHistory; // this is being activated only upon first interactive call.
public:
	PositiveRefinement();
	virtual ~PositiveRefinement();
	void initialize(InputSet *cs);
	void finalize();
	bool isEmpty() const;
	ResolvingSequence *getNextResolvingSequence();	
	std::map<Clause*, int> getIndicesMap();
	std::string getIndicesMessage();
	ResolvingSequence *getResolvingSequenceFromIndices(std::vector<int>);
	void setEventPerformed(InputSetEvent ise);
	
private:
	void insertClause(Clause*);
	bool isPositive(Clause*);
	void setNextIndices();
};

#endif /*POSITIVEREFINEMENT_H_*/
