/*
 * ProofEnvironment.h
 *
 *  Created on: 30-Oct-2008
 *      Author: shaolin
 */

#ifndef PROOFENVIRONMENT_H_
#define PROOFENVIRONMENT_H_

#include "ATPException.h"

#include "Refinement.h"
#include "ResolvingUnit.h"
#include "ResolventPostProcessor.h"
#include "InputSet.h"
#include "Resolvent.h"
#include "ClausesInput.h"
#include "DeductionEngine.h"
#include "ManagedRefinement.h"
#include <string>
#include "NodePrint.h"

class DeductionEngine;
class Refinement;
class ManagedRefinement;
class ResolvingUnit;
class ClausesInput;
class ResolventPostProcessor;
class NodePrint;

class ProofEnvironment {

private:
	Refinement *refinement;
	ResolvingUnit *resolver;
	int timeout;
	std::set<ResolventPostProcessor*, PostProcessorPriorityCmp> postProcessors;
	ClausesInput *clausesInput;
	DeductionEngine *engine;
	ManagedRefinement *managedRef;
    std::string outputFilename;
    NodePrint *printer;
	std::string filename;

public:
	ProofEnvironment();
	virtual ~ProofEnvironment();

	void setRefinement(Refinement *r);
	Refinement* getRefinement();
	void setResolvingUnit(ResolvingUnit *r);
	ResolvingUnit* getResolvingUnit();
	void setTimeout(unsigned int);
	unsigned int getTimeout();
	void addPostProcessor(ResolventPostProcessor*);
	std::set<ResolventPostProcessor*, PostProcessorPriorityCmp> &getPostProcessors();
	void setClausesInput(ClausesInput*);
	ClausesInput *getClausesInput();
	void setDeductionEngine(DeductionEngine *e);
	DeductionEngine *getDeductionEngine();
	void setManagedRefinement(ManagedRefinement *m);
	ManagedRefinement *getManagedRefinement();
	void setOutputFilename(std::string &s);
	std::string &getOutputFilename();
	void setNodePrint(NodePrint *n);
	NodePrint *getNodePrint();
	void setInputFilename(const std::string &s);
	std::string &getInputFilename();
};

#endif /* PROOFENVIRONMENT_H_ */
