#include "ProofNode.h"
#include <iostream>
using namespace std;
ProofNode::ProofNode()
{
    parents = new vector<ProofNode*>();
    children = new vector<ProofNode*>();
}

ProofNode::~ProofNode()
{
    // may be deleted several times so check first if not deleted already
    if (parents != NULL)
    {
        for (vector<ProofNode*>::const_iterator it = parents->begin(); it != parents->end(); ++it)
        {
            delete *it;
        }
    }
    if (children != NULL)
    {
        for (vector<ProofNode*>::const_iterator it = children->begin(); it != children->end(); ++it)
        {
            delete *it;
        }
    }
    delete parents;
    delete children;
}

void ProofNode::addChild(ProofNode* n)
{
    children->push_back(n);
}
    
void ProofNode::addParent(ProofNode* n)
{
    parents->push_back(n);
    n->addChild(this);
}

set<ProofNode*> ProofNode::getParentsOfType(const type_info &ti)
{
	set<ProofNode*> ret;
	for (vector<ProofNode*>::const_iterator it = parents->begin(); it != parents->end();++it)
	{
		ProofNode *pn = *it;
		if (typeid(*pn) == ti)
		{
			ret.insert(pn);
		}
		else
		{
			set<ProofNode*> v = pn->getParentsOfType(ti);
			ret.insert(v.begin(), v.end());
		}
	}
	return ret;
}

set<ProofNode*> ProofNode::getChildrenOfType(const type_info &ti)
{
	set<ProofNode*> ret;
	for (vector<ProofNode*>::const_iterator it = children->begin(); it != children->end();++it)
	{
		ProofNode *pn = *it;
		if (typeid(*pn) == ti)
		{
			ret.insert(pn);
		}
		else
		{
			set<ProofNode*> v = pn->getChildrenOfType(ti);
			ret.insert(v.begin(), v.end());
		}
	}
	return ret;
}

vector<ProofNode*> *ProofNode::getChildren()
{
	return children;
}

string ProofNode::toString()
{
	return "Proof Node";
}

void ProofNode::print(NodePrint &printer)
{
    printer.beginNode(*this);
    printer.beginChildren(*this);
    for (vector<ProofNode*>::const_iterator it = parents->begin(); it != parents->end();++it)
    {
        (*it)->print(printer);
    }
    printer.endChildren(*this);
    printer.endNode(*this);
}

bool ProofNode::hasParents()
{
    return !parents->empty();
}