#include "SimpleRefinement.h"

using namespace std;
using namespace NS_PROOFLIB;

SimpleRefinement::SimpleRefinement()
{
	index = 0;
}

SimpleRefinement::~SimpleRefinement()
{
}

void SimpleRefinement::deletePreviousData()
{
    ds.clear();
}

/*create all possible combinations of literal labeled clauses.
 *create variants of each clause
 *Factorization and other things are redundant between applications
 *on the same clauses
 */
void SimpleRefinement::initialize(InputSet *csin)
{
    deletePreviousData();
    typedef set<Clause*>::const_iterator LCI;
    cs = csin;
    set<Clause * > *cs1 = cs->getClauseSet();
    // iterate on all clauses
    for (LCI i1 = cs1->begin(); i1 != cs1->end(); ++i1)
    {
            Clause *c1 = *i1;
            set<Clause * > *cs2 = cs->getClauseSet();
            // iterate again on all clauses
            for (LCI i2 = cs2->begin(); i2 != cs2->end(); ++i2)
            {
                Clause *c2 = *i2;
                // if the two clauses are different
                if (c1 < c2)
                {
                    ds.push_back(new pair<Clause*, Clause*>(c1,c2));
                }
            }	
    }
    curInputSet.clear();
    curInputSet.insert(curInputSet.begin(), cs->getClauseSet()->begin(), cs->getClauseSet()->end());
}

void SimpleRefinement::finalize()
{
}

bool SimpleRefinement::isEmpty() const
{
	return ds.size() <= (unsigned int) index;
}

ResolvingSequence *SimpleRefinement::getNextResolvingSequence()
{
    vector<Clause*> rsl;
    pair<Clause*, Clause*> *p = ds.at(index++);
    rsl.push_back(p->first);
    rsl.push_back(p->second);
    ResolvingSequence *rs = new ResolvingSequence(new SimpleStrategy()); 
    rs->initialize(rsl);
    return rs;
}

void SimpleRefinement::setEventPerformed(InputSetEvent ise)
{
	set<Clause * > *cs1 = cs->getClauseSet();
	switch (ise.getType())
	{
	case InputSetEvent::CLAUSE_ADDED:
	    typedef set<Clause*>::const_iterator LCI;
			    
	    // iterate on all clauses
	    for (LCI i1 = cs1->begin(); i1 != cs1->end(); ++i1)
	    {
	        Clause *c1 = *i1;        
	        ds.push_back(new pair<Clause*, Clause*>(c1,ise.getClause()));
	    }
	    curInputSet.push_back(ise.getClause());
	    break;
	case InputSetEvent::CLAUSE_DELETED:
		throw new ATPException("deletion not implemented yet in simple refinement");
	case InputSetEvent::CLAUSE_REFUSED:
		//nothing to do
		break;
	}
}

map<Clause*, int> SimpleRefinement::getIndicesMap()
{
	map<Clause*, int> ret;
	int i=0;
	for (vector<Clause*>::const_iterator it = curInputSet.begin(); it != curInputSet.end(); ++it)
	{
		ret[*it] = i;
		i++;
	}
	return ret;
}

string SimpleRefinement::getIndicesMessage()
{
	return "Please choose two clauses to process: ";
}

ResolvingSequence *SimpleRefinement::getResolvingSequenceFromIndices(vector<int> l)
{
	Clause *c1 = curInputSet[l[0]];
	Clause *c2 = curInputSet[l[1]];
	// find the specific pair in ds
	int i = 0;
	for (std::vector<std::pair<Clause*, Clause*>*>::const_iterator it = ds.begin(); it != ds.end(); ++it)
	{
		pair<Clause*, Clause*>* p = *it;
		if ( (c1 == p->first && c2 == p->second) || (c2 == p->first && c1 == p->second) )
		{
			// if the current location is before index, then we have already visited this pair
			if (i < index)
			{
				throw new RefinementException("The chosen clauses were already resolved upon");
			}
			// swap the chosen pair with the pair at the current index
			else
			{
				ds[i] = ds[index];
				ds[index] = p;
			}
			break;
		}
		i++;
	}
	return getNextResolvingSequence();
}
