#ifndef UNIFIER_H_
#define UNIFIER_H_

#include "Term.h"
#include "Variable.h"
#include "Constant.h"
#include "Function.h"
#include "ATPException.h"

#include "InformativeProofNode.h"
#include "ClauseContext.h"

#include <string>

class UnificationException : public ATPException 
{
public :
	UnificationException(const std::string msg) : ATPException(msg) {}
};

class Unifier : public InformativeProofNode, public ClauseContext
{
public:
	enum Status {OK, OccurCheck, SymbolClash}; 
	
private:
	Status status;
	
protected:
	void setStatus(Status s);
	
public:
	Unifier();
	Unifier(Unifier&);
	virtual ~Unifier();
	Status getStatus();
	/* merges the two context as they must be disjoint
	 */
	void initialize(ClauseContext *cc1, ClauseContext *cc2);
	void initialize(ClauseContext *cc);
	/*
	 * similar to merge but confirm that if a variable already exists, then it is
	 * mapped to the same term
	 */
	void merge(ClauseContext *c);
	virtual void unify(NS_PROOFLIB::Term *, NS_PROOFLIB::Term *) = 0;
	std::string toString();
	std::string getStatusString();
        virtual void print(NodePrint&);
};

#endif /*UNIFIER_H_*/
