#include "UnitRefinement.h"

using namespace std;
using namespace NS_PROOFLIB;

UnitRefinement::UnitRefinement()
{
	exhausted = 0;
}

UnitRefinement::~UnitRefinement()
{
}

void UnitRefinement::initialize(InputSet *cs)
{
	set<Clause*> *clauses = cs->getClauseSet();
	for (set<Clause*>::const_iterator it = clauses->begin(); it != clauses->end(); ++it)
	{
		Clause *clause = *it;
		insertClause(clause);
	}
}

void UnitRefinement::finalize()
{
	
}

bool UnitRefinement::isEmpty() const
{
	// is empty when there are no more unit clauses
	return elements.empty();
}

ResolvingSequence *UnitRefinement::getNextResolvingSequence()
{
    pair<pair<Clause*,bool>,pair<unsigned int,unsigned int*> > p = elements.front();
    elements.pop_front();
    pair<Clause*, pair<unsigned int, unsigned int*> > newp = getClauseAtIndex(p.second.first, p.second.second, p.first.second);
    if (newp.first == NULL)
    {
    	// no matching clause can be found
    	// insert the current p again at the end of the stack and return next item
    	// in case all units are in this situation, abort!
    	// checking this by increasing by 1 each time and resetting when an element is returned
    	if (elements.empty() || ++exhausted == elements.size())
    	{
    		// means all units were exhausted
    		// empty elements and return NULL, but first release memory
    		typedef list<pair<pair<Clause*, bool>, pair<unsigned int,unsigned int*> > >::const_iterator IT;
    		for (IT it = elements.begin(); it != elements.end(); ++it)
    		{
    			delete[] (*it).second.second;
    		}
    		elements.clear();
    		return NULL;
    	}
    	else
    	{
    		// reinsert the element at the end and return next
    		elements.push_back(p);
    		return getNextResolvingSequence();
    	}
    }
    else
    {
    	// first reset exhausted
    	exhausted = 0;
    	// insert the next index at the end
    	elements.push_back(pair<pair<Clause*,bool>, pair<unsigned int,unsigned int*> >(p.first, newp.second));
    	// return sequence
    	vector<Clause*> rsl;
		rsl.push_back(p.first.first);
		rsl.push_back(newp.first);
		ResolvingSequence *rs = new ResolvingSequence(new SimpleStrategy()); 
		rs->initialize(rsl);
		return rs;
    }
}

/* is not optimized as I am currently do not remove user chosen pair of clauses
 * from the algorithm, so the refinement will return it again at some phase
 * it builds a unique id for each clause such that the clause can be restored
 * from and the check for unit can be done easily as well
 * we take the size of all vectors to be A
 * and each clause index is transformed into index for unit_pos
 * A + index for unit_neg and so on
 */
map<Clause*, int> UnitRefinement::getIndicesMap()
{
	unsigned int totalSize = unit_pos.size() + unit_neg.size() + pos.size() + neg.size() + both.size();
	map<Clause*, int> ret;
	int coefficient = 0;
	
	typedef vector<Clause*>::const_iterator IT;
	
	int i = 0;
	for (IT it = unit_pos.begin(); it != unit_pos.end(); ++it)
	{
		ret[*it] = i;
		i++;
	}
	
	coefficient = totalSize;
	i = 0;
	for (IT it = unit_neg.begin(); it != unit_neg.end(); ++it)
	{
		ret[*it] = i+coefficient;
		i++;
	}
	
	coefficient += totalSize;
	i = 0;
	for (IT it = pos.begin(); it != pos.end(); ++it)
	{
		ret[*it] = i+coefficient;
		i++;
	}
	
	coefficient += totalSize;
	i = 0;
	for (IT it = neg.begin(); it != neg.end(); ++it)
	{
		ret[*it] = i+coefficient;
		i++;
	}

	coefficient += totalSize;
	i = 0;
	for (IT it = both.begin(); it != both.end(); ++it)
	{
		ret[*it] = i+coefficient;
		i++;
	}
	
	return ret;
}

string UnitRefinement::getIndicesMessage()
{
	return "Please choose two clauses to process: ";
}

ResolvingSequence *UnitRefinement::getResolvingSequenceFromIndices(vector<int> l)
{
	unsigned int index1 = l[0];
	unsigned int index2 = l[1];
	
	unsigned int totalSize = unit_pos.size() + unit_neg.size() + pos.size() + neg.size() + both.size();
	
	if (index1 >= totalSize * 2 && index2 >= totalSize * 2)
	{
		throw new RefinementException("At least one clause must be a unit clause!");
	}
	
	Clause *c1, *c2;
	
	if (index1 < totalSize)
	{
		c1 = unit_pos[index1];
	}
	else if (index1 < totalSize * 2)
	{
		c1 = unit_neg[index1-totalSize];
	}
	else if (index1 < totalSize * 3)
	{
		c1 = pos[index1-totalSize*2];
	}
	else if (index1 < totalSize * 4)
	{
		c1 = neg[index1-totalSize*3];
	}
	else if (index1 < totalSize * 5)
	{
		c1 = both[index1-totalSize*4];
	}
	else
	{
		throw new ATPException("Illegal index retrieved from UserCommand in UnitRefinement");
	}
	
	if (index2 < totalSize)
	{
		c2 = unit_pos[index2];
	}
	else if (index2 < totalSize * 2)
	{
		c2 = unit_neg[index2-totalSize];
	}
	else if (index2 < totalSize * 3)
	{
		c2 = pos[index2-totalSize*2];
	}
	else if (index2 < totalSize * 4)
	{
		c2 = neg[index2-totalSize*3];
	}
	else if (index2 < totalSize * 5)
	{
		c2 = both[index2-totalSize*4];
	}
	else
	{
		throw new ATPException("Illegal index retrieved from UserCommand in UnitRefinement");
	}

	
	vector<Clause*> rsl;
	rsl.push_back(c1);
	rsl.push_back(c2);
	ResolvingSequence *rs = new ResolvingSequence(new SimpleStrategy()); 
	rs->initialize(rsl);
	return rs;
}

void UnitRefinement::insertClause(Clause* c)
{
	int queue = insertToVectors(c);
	unsigned int *initial_array = new unsigned int[3];
        initial_array[0] = 0;initial_array[1] = 0;initial_array[2] = 0;

	switch (queue)
	{
	case 0:
		// is unit pos
		elements.push_front(pair<pair<Clause*,bool>,pair<unsigned int,unsigned int*> >(pair<Clause*, bool>(c, true), pair<unsigned int,unsigned int*>(1,initial_array)));
		break;
	case 1:
		// is unit neg
		elements.push_front(pair<pair<Clause*,bool>,pair<unsigned int,unsigned int*> >(pair<Clause*, bool>(c, false), pair<unsigned int,unsigned int*>(0,initial_array)));
		break;
	default:
		// do nothing
		;
	}
}

int UnitRefinement::insertToVectors(Clause* c)
{
	vector<Term*> pos_terms = c->getPositiveLiterals();
	vector<Term*> neg_terms = c->getNegativeLiterals();
	
	if (neg_terms.empty())
	{
		if (pos_terms.size() == 1)
		{
			// unit pos
			unit_pos.push_back(c);
			return 0;
		}
		else
		{
			// pos
			pos.push_back(c);
			return 2;
		}	
	}
	else if (pos_terms.empty())
	{
		if (neg_terms.size() == 1)
		{
			// unit neg
			unit_neg.push_back(c);
			return 1;
		}
		else
		{
			//neg
			neg.push_back(c);
			return 3;
		}
	}
	else
	{
		// both
		both.push_back(c);
		return 4;
	}
}

pair<Clause*, pair<unsigned int, unsigned int*> > UnitRefinement::getClauseAtIndex(unsigned int queue, unsigned int* indices, bool isPos)
{
	// indices is int[3]
	
	// First get the clause
	Clause *c = NULL;
	int numOfLoops = 0;
	unsigned int nextQueue;
	unsigned int queueCode;
	if (isPos)
	{
		// is positive then complement clause must contain a negative literal
		while (c == NULL && numOfLoops < 3)
		{
			switch (queue)
			{
			case 1:
				queueCode = 0;
				if (indices[queueCode] < unit_neg.size())
				{
					c = unit_neg[indices[queueCode]];
				}
				else
				{
					queue = 3;
				}
				nextQueue = 4;
				break;
			case 3:
				queueCode = 1;
				if (indices[queueCode] < neg.size())
				{
					c = neg[indices[queueCode]];
				}
				else
				{
					queue = 4;
				}
				nextQueue = 1;
				break;
			case 4:
				queueCode = 2;
				if (indices[queueCode] < both.size())
				{
					c = both[indices[queueCode]];
				}
				else
				{
					queue = 1;
				}
				nextQueue = 3;
				break;
			}	
			numOfLoops++;
		}
	}
	else
	{
		// is negative then complement clause must contain a positive literal
		while (c == NULL && numOfLoops < 3)
		{
			switch (queue)
			{
			case 0:
				queueCode = 0;
				if (indices[queueCode] < unit_pos.size())
				{
					c = unit_pos[indices[queueCode]];
				}
				else
				{
					queue = 2;
				}
				nextQueue = 4;
				break;
			case 2:
				queueCode = 1;
				if (indices[queueCode] < pos.size())
				{
					c = pos[indices[queueCode]];
				}
				else
				{
					queue = 4;
				}
				nextQueue = 0;
				break;
			case 4:
				queueCode = 2;
				if (indices[queueCode] < both.size())
				{
					c = both[indices[queueCode]];
				}
				else
				{
					queue = 0;
				}
				nextQueue = 2;
				break;
			}	
			numOfLoops++;
		}
	}
	
	// now create the new indices and queue if c exists, otherwise return failure
	if (c != NULL)
	{
		// can reuse indices memory
		// first set queue to next queue
		queue = nextQueue;
		// now increasethe index
		indices[queueCode]++;
		// now return the new element
		return pair<Clause*, pair<unsigned int, unsigned int*> >(c, pair<unsigned int, unsigned int*>(queue, indices));
	}
	else
	{
		return pair<Clause*, pair<unsigned int, unsigned int*> >(NULL, pair<unsigned int, unsigned int*>(queue, indices));
	}
}

void UnitRefinement::setEventPerformed(InputSetEvent ise)
{
	switch (ise.getType())
	{
	case InputSetEvent::CLAUSE_ADDED:
		insertClause(ise.getClause());
	    break;
	case InputSetEvent::CLAUSE_DELETED:
		throw new ATPException("deletion not implemented yet in unit refinement");
	case InputSetEvent::CLAUSE_REFUSED:
		//nothing to do
		break;
	}
}
