#ifndef UNITREFINEMENT_H_
#define UNITREFINEMENT_H_

#include "ManagedRefinement.h"

#include <list>
#include <vector>

/*
 * This refinement contains 5 sets, unit_pos, unit_neg, pos, neg, both.
 * It has a fair policy using a queue which gets all unit clauses and for each
 * an index is attached marking the place of the matched item. By keeping an order
 * between queues (as specified above) we ensure each pair will be selected once only.
 * TODO IMPORTANT: as equalities are considered as positive literals and I dont match
 * positive literals together, then there will be no attempt to match positive equalities
 * positive clauses!!!
 */
class UnitRefinement : public ManagedRefinement
{
	// contains all elements and maps them to current vector and current index in 
	// each of the vectors
	std::list<std::pair<std::pair<Clause*, bool>, std::pair<unsigned int,unsigned int*> > > elements;
	std::vector<Clause*> unit_pos; // vector 0
	std::vector<Clause*> unit_neg; // vector 1
	std::vector<Clause*> pos; // vector 2
	std::vector<Clause*> neg; // vector 3
	std::vector<Clause*> both; // vector 4

	//int unit_pos_order [3];// = {1,3,4};
	//int unit_neg_order [3];// = {0,2,4};
	
	// counter to make sure we ar not infinitely looping on units when they
	// were exhausted
	unsigned int exhausted;
	
public:
	UnitRefinement();
	virtual ~UnitRefinement();
	void initialize(InputSet *cs);
	void finalize();
	bool isEmpty() const;
	ResolvingSequence *getNextResolvingSequence();	
	std::map<Clause*, int> getIndicesMap();
	std::string getIndicesMessage();
	ResolvingSequence *getResolvingSequenceFromIndices(std::vector<int>);
	void setEventPerformed(InputSetEvent ise);
	
private:
	void insertClause(Clause*);
	int insertToVectors(Clause*);
	std::pair<Clause*, std::pair<unsigned int, unsigned int*> > getClauseAtIndex(unsigned int, unsigned int*, bool);
};

#endif /*UNITREFINEMENT_H_*/
