#include "SetupParser.h"

#include <iostream>
#include <cstdlib>

using namespace std;
using namespace NS_PROOFLIB;

SetupParser::SetupParser()
  : xmlpp::SaxParser()
{
	env = new ExtendedProofEnvironment();
}

SetupParser::~SetupParser()
{
}

void SetupParser::on_start_document()
{
}

void SetupParser::on_end_document()
{
}

void SetupParser::on_start_element(const Glib::ustring& name,
                                   const AttributeList& attributes)
{
	const char *attrName[2];
	const char *attrValue[2];
	int numOfAttr = 0;
  	for(xmlpp::SaxParser::AttributeList::const_iterator iter = attributes.begin(); iter != attributes.end(); ++iter)
  	{
    		attrName[numOfAttr] = iter->name.c_str();
		attrValue[numOfAttr] = iter->value.c_str();
		numOfAttr++;
  	}

        if (numOfAttr > 0)
        {
            string nm(attrName[0]);
            string val(attrValue[0]);

            if (numOfAttr > 2)

            {
                    throw xmlpp::exception("Illegal number of attributes");
            }

            if (name == "timeout")
            {
                    if (nm != "value")
                    {
                            throw xmlpp::exception("Illegal name of attribute");
                    }
                    int timeout = std::atoi(attrValue[0]);
                    env->setTimeout(timeout);
            }

            else if (name == "UserCommand")
            {
                    if (nm != "class")
                    {
                            throw xmlpp::exception("Illegal name of attribute");
                    }
                    if (val == "PromptUserCommand")
                    {
                            userCommand = new PromptUserCommand();
                            userCommand->setEnvironment(env);
                    }
                    else
                    {
                            throw xmlpp::exception("Unknown UserCommand class");
                    }
            }

            else if (name == "DeductionEngine")
            {
                    if (nm != "class")
                    {
                            throw xmlpp::exception("Illegal name of attribute");
                    }
                    if (val == "BinaryDeductionEngine")
                    {
                            env->setDeductionEngine(new BinaryDeductionEngine());
                    }
                    else
                    {
                            throw xmlpp::exception("Unknown DeductionEngine class");
                    }
            }

            else if (name == "VariableNameGenerator")
            {
                    if (nm != "class")
                    {
                            throw xmlpp::exception("Illegal name of attribute");
                    }
                    if (val == "SimpleVariableNameGenerator")
                    {
                            env->setVariableNameGenerator(new SimpleVariableNameGenerator());
                    }
                    else
                    {
                            throw xmlpp::exception("Unknown VariableNameGenerator class");
                    }
            }

            else if (name == "UnificationAlgorithm")
            {
                    if (nm != "class")
                    {
                            throw xmlpp::exception("Illegal name of attribute");
                    }
                    if (val == "SimpleUnificationAlgorithm")
                    {

                    	env->setUnificationAlgorithm(new SimpleUnificationAlgorithm());
                    }
                    else
                    {
                            throw xmlpp::exception("Unknown UnificationAlgorithm class");
                    }
            }

            else if (name == "ParaModulator")
            {
            	ParaModulator *pmod;
                    if (nm != "class")
                    {
                            throw xmlpp::exception("Illegal name of attribute");
                    }
                    if (val == "SimpleParaModulator")
                    {
                            pmod = new SimpleParaModulator();
                            pmod->setUnificationAlgorithm(env->getUnificationAlgorithm());
                    }
                    else if (val == "BFSParaModulator")
                    {
                            pmod = new BFSParaModulator();
                            pmod->setUnificationAlgorithm(env->getUnificationAlgorithm());
                    }
                    else if (val == "NoParaModulator")
                    {
                            pmod = new NoParaModulator();
                    }
                    else
                    {
                            throw xmlpp::exception("Unknown ParaModulator class");
                    }
                    env->setParaModulator(pmod);
            }

            else if (name == "ClausesInput")
            {
                    if (nm != "class")
                    {
                            throw xmlpp::exception("Illegal name of attribute");
                    }
                    /*
                     * As it has the same classes names as Prover9Driver, only one
                     * module can be loaded at a time, if you wish to change to this
                     * module, it must be changed in the makefiles of the root makefile,
                     * the makefile inside Prover/src and in the code itself of SetupParser.h
                     * and SetupParser.cpp (this file)
                     */
                    /*if (val == "ClausesDriver")
                    {
                        ClausesDriver *driver = new ClausesDriver(inputfile);
                        ProofDatabase *m_pdb  = new ProofDatabase();
                        variableNameGenerator->setProofDatabase(m_pdb);
                        driver->pdb = m_pdb;
                        driver->vng = variableNameGenerator;
                        clausesInput = driver;
                    }*/
                    /*if (val == "Prover9Driver")
                    {
                    	Prover9Driver *driver = new Prover9Driver(env);
                        ProofDatabase *m_pdb  = new ProofDatabase();
                        env->getVariableNameGenerator()->setProofDatabase(m_pdb);
                        driver->pdb = m_pdb;
                        driver->vng = env->getVariableNameGenerator();
                        env->setClausesInput(driver);
                    }*/
                    if (val == "SpiritParser")
					{
						ProofDatabase *m_pdb  = new ProofDatabase();
						env->getVariableNameGenerator()->setProofDatabase(m_pdb);
						env->setProofDatabase(m_pdb);
						SpiritParser *parser = new SpiritParser(env);
						env->setClausesInput(parser);
					}
                    else
                    {
                            throw xmlpp::exception("Unknown ClausesInput class");
                    }
            }

            else if (name == "ResolvingUnit")
            {
                    if (nm != "class")
                    {
                            throw xmlpp::exception("Illegal name of attribute");
                    }
                    if (val == "UnificationResolver")
                    {
                        env->setResolvingUnit(new UnificationResolver(env));
                    }
                    else
                    {
                            throw xmlpp::exception("Unknown ResolvingUnit class");
                    }
            }

            else if (name == "Factorizer")
            {
                    if (nm != "class")
                    {
                            throw xmlpp::exception("Illegal name of attribute");
                    }
                    if (val == "ResolvedLiteralFactorizer")
                    {
                        env->setFactorizer(new ResolvedLiteralFactorizer());
                    }
                    else if (val == "NoFactorizer")
                    {
                    	env->setFactorizer(new NoFactorizer());
                    }
                    else
                    {
                            throw xmlpp::exception("Unknown Factorizer class");
                    }
            }

            else if (name == "ManagedRefinement")
            {
                    if (nm != "class")
                    {
                            throw xmlpp::exception("Illegal name of attribute");
                    }
                    if (val == "UnitRefinement")
                    {
                           env->setManagedRefinement(new UnitRefinement());
                    }
                    if (val == "PositiveRefinement")
                    {
                    	env->setManagedRefinement(new PositiveRefinement());
                    }
                    else
                    {
                            throw xmlpp::exception("Unknown ManagedRefinement class");
                    }
            }

            else if (name == "Refinement")
            {
                    if (nm != "class")
                    {
                            throw xmlpp::exception("Illegal name of attribute");
                    }
                    if (val == "UnitRefinement")
                    {
                            env->setRefinement(new UnitRefinement());
                    }
                    else if (val == "PositiveRefinement")
                    {
                    	env->setRefinement(new PositiveRefinement());
                    }
                    else if (val == "InteractiveRefinement")
                    {
                            InteractiveRefinement *irefinement = new InteractiveRefinement(env);
                            irefinement->setUserCommand(userCommand);
                            env->setRefinement(irefinement);
                    }
                    else if (val == "SimpleRefinement")
                    {
                        env->setRefinement(new SimpleRefinement());
                    }
                    else
                    {
                            throw xmlpp::exception("Unknown Refinement class");
                    }
            }

            else if (name == "SubsumptionAlgorithm")
            {
                    if (nm != "class")
                    {
                            throw xmlpp::exception("Illegal name of attribute");
                    }
                    if (val == "StillmanSubsumptionAlgorithm")
                    {
                            env->setSubsumptionAlgorithm(new StillmanSubsumptionAlgorithm(env));
                            env->setUnificationAlgorithmForSubsumption(env->getUnificationAlgorithm()); // subsumption uses the main unification algorithm
                    }
                    else
                    {
                            throw xmlpp::exception("Unknown SubsumptionAlgorithm class");
                    }
            }

            else if (name == "PostProcessor")
            {
					string nm2(attrName[1]);
                    ResolventPostProcessor *rpp;
                    if (nm == "class")
                    {
                            if (val == "SimpleInsertPostProcessor")
                            {
                                    rpp = new SimpleInsertPostProcessor();
                                    // locate forward subsumption if exists and add it as listener to this rpp
                                    typedef std::set<ResolventPostProcessor*, PostProcessorPriorityCmp>::const_iterator LCI;
                                    ResolventPostProcessor *rppit;
									for (LCI i = env->getPostProcessors().begin(); i != env->getPostProcessors().end(); ++i)
									{
											ResolventPostProcessor *rppit = *i;
											// check if it is of class ForwardSubsumptionPostProcessor
											if (typeid(*rppit) == typeid(ForwardSubsumptionPostProcessor))
											{
												ForwardSubsumptionPostProcessor *f = dynamic_cast<ForwardSubsumptionPostProcessor*>(rppit);
												rpp->addInputSetEventListener(f);
											}
									}

                            }
                            else if (val == "TautologyEliminationPostProcessor")
                            {
                                    rpp = new TautologyEliminationPostProcessor();
                            }
                            else if (val == "ForwardSubsumptionPostProcessor")
                            {
                            	ForwardSubsumptionPostProcessor *forwardSubsumptionPostProcessor = new ForwardSubsumptionPostProcessor();
								forwardSubsumptionPostProcessor->setSubsumptionAlgorithm(env->getSubsumptionAlgorithm());
								rpp = forwardSubsumptionPostProcessor;
                            }
                            else
                            {
                                    throw xmlpp::exception("Unknown PostProcessor class");
                            }
                            rpp->addInputSetEventListener(env->getRefinement());
                    }
                    if (nm2 == "priority")
                    {
                            int priority = std::atoi(attrValue[1]);
                            rpp->priority = priority;
                    }
                    env->addPostProcessor(rpp);
            }

            else if (name == "Inputfile")
            {
                    if (nm != "value")
                    {
                            throw xmlpp::exception("Illegal name of attribute");
                    }
                    env->setInputFilename(val);
            }

            else if (name == "Outputfile")
            {
                    if (nm != "value")
                    {
                            throw xmlpp::exception("Illegal name of attribute");
                    }
                    env->setOutputFilename(val);
            }
            else if (name == "NodePrint")
            {
                    if (nm != "class")
                    {
                            throw xmlpp::exception("Illegal name of attribute");
                    }
                    if (val == "LatexNodePrint")
                    {
                            env->setNodePrint(new LatexNodePrint(env));
                    }
                    else  if (val == "CEResXMLNodePrint")
                    {
                            env->setNodePrint(new CEResXMLNodePrint(env));
                    }
                    else
                    {
                            throw xmlpp::exception("Unknown NodePrint class");
                    }
            }

            else
            {
                    throw xmlpp::exception("Illegal name of element");
            }
        }
}

void SetupParser::on_end_element(const Glib::ustring& name)
{
  //std::cout << "on_end_element()" << std::endl;
}

void SetupParser::on_characters(const Glib::ustring& text)
{
  //std::cout << "on_characters(): " << text << std::endl;
}

void SetupParser::on_comment(const Glib::ustring& text)
{
  //std::cout << "on_comment(): " << text << std::endl;
}

void SetupParser::on_warning(const Glib::ustring& text)
{
  std::cout << "on_warning(): " << text << std::endl;
}

void SetupParser::on_error(const Glib::ustring& text)
{
  std::cout << "on_error(): " << text << std::endl;
}

void SetupParser::on_fatal_error(const Glib::ustring& text)
{
  std::cout << "on_fatal_error(): " << text << std::endl;
}


