%{ /* -*- C++ -*- */
# include <cstdlib>
# include <errno.h>
# include <limits.h>
# include <string>
# include "Prover9Driver.h"
# include "y.tab.h"

/* Work around an incompatibility in flex (at least versions
2.5.31 through 2.5.33): it generates code that does
not conform to C89. See Debian bug 333231
<http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=333231>. */
# undef yywrap
# define yywrap() 1

/* By default yylex returns int, we use token_type.
Unfortunately yyterminate by default returns 0, which is
not of token_type. */
#define yyterminate() return token::END
%}

%option noyywrap nounput batch debug

const [a-c][0-9]*
all_id [d-zA-Z0-9\+\*_{}<]+
comment \x25.*
blank [ \t]*

%{
# define YY_USER_ACTION yylloc->columns (yyleng);
%}

%%

%{
yylloc->step ();
%}
{blank}+ yylloc->step ();
[\n]+ yylloc->lines (yyleng); yylloc->step ();

%{
typedef yy::ClausesParser::token token;
%}

"(" return token::OBRACE;
")" return token::EBRACE;
"{" return token::OCBRACE;
"}" return token::ECBRACE;
"," return token::COMMA;
":-" return token::IMPLIED;
"." return token::DOT;
"=" return token::EQ;
"\"" return token::QT;
"-" return token::NEG;
"|" return token::OR;

{all_id} yylval->sval = new std::string (yytext); return token::CHAR;
{const} yylval->sval = new std::string (yytext); return token::CONST;
{comment} yylval->sval = new std::string (yytext); return token::COMMENT;
. driver.error (*yylloc, "invalid character");

%%

void Prover9Driver::scan_begin ()
{
	yy_flex_debug = trace_scanning;
	if (!(yyin = fopen (file.c_str (), "r")))
	error (std::string ("cannot open ") + file);
}

void Prover9Driver::scan_end ()
{
	fclose (yyin);
}
