/*
 * Prover9Grammer.h
 *
 *  Created on: 05-Nov-2008
 *      Author: shaolin
 */

//#define BOOST_SPIRIT_DEBUG

#ifndef PROVER9GRAMMER_H_
#define PROVER9GRAMMER_H_

#include "SpiritProofEnvironmentExtension.h"
#include "Result.h"

#include <boost/spirit/core.hpp>
#include <boost/spirit/phoenix/primitives.hpp>
#include <boost/spirit/phoenix/operators.hpp>
#include <boost/spirit/phoenix/functions.hpp>
#include <boost/spirit/iterator/file_iterator.hpp>
#include <iostream>
#include <vector>
#include <string>

////////////////////////////////////////////////////////////////////////////
using namespace std;
using namespace boost::spirit;

typedef char                    char_t;
typedef file_iterator<char_t>   iterator_t;
typedef scanner<iterator_t>     scanner_t;
typedef rule<scanner_t>         rule_t;

////////////////////////////////////////////////////////////////////////////
//
//  Semantic actions
//
////////////////////////////////////////////////////////////////////////////

namespace
{

void
    bbb(char const* first, char const* last)
    {
        std::string str(first, last);
        std::cout << str << std::endl;
    }

void
    bb(char const first)
    {
        std::cout << first << std::endl;
    }
}

namespace Private
{
	struct InsertClause;
	struct CreateClause;
	struct InsertLiteral;
	struct CreateFunctionName;
	struct CreateFunction;
	struct CreateVariable;
	struct CreateConstant;
	struct AddTerm;
}



class Prover9Grammer: public grammar<Prover9Grammer>
{

public:
	Result &result;

	Prover9Grammer(Result &r) : result( r ) {}
	virtual ~Prover9Grammer() {}

	template <typename ScannerT>
	    struct definition
	    {
	        definition(Prover9Grammer const &self)
	        {
	            clause_set
	                =   clause [Private::InsertClause( self.result )] >> (clause_set | end_p)
	                ;

				clause
					=	(literal_vector [Private::CreateClause( self.result )] >> ch_p('.') || ('%' >> comment >> eol_p)) ;

				literal_vector
					=	('-' >> term [Private::InsertLiteral( self.result , false)] || ('|' >> literal_vector))
					|	(term [Private::InsertLiteral( self.result , true)] || ('|' >> literal_vector))
					;

				term
					=	function
					|	constant
					|	variable
					;

				function
					= 	//('\"' >> chr [Private::CreateFunctionName( self.result )] >> '\"' >> '(' >> term_list >> ')') [Private::CreateFunction(self.result)]
						(eql [Private::CreateFunctionName( self.result)] >> '(' >> term_list >> ')') [Private::CreateFunction( self.result)]
					|	(cnst [Private::CreateFunctionName( self.result)] >> '(' >> term_list >> ')') [Private::CreateFunction( self.result)]
					;

				variable
					=	cnst [Private::CreateVariable( self.result)]
					;

				constant
					=	//('\"' >> chr [Private::CreateConstant( self.result)] >> '\"')
						vrb [Private::CreateConstant( self.result)]
					;

				cnst
					= chr >> wrd
					;

				vrb
					= range_p('a','c') >> wrd
					;

				wrd
					= (*alnum_p)
					;

				chr
					=	range_p('d','z')
					;

				eql
					= ch_p('=')
					;

				term_list
					= 	(term [Private::AddTerm( self.result)] || ',' >> term_list)
					;

				comment
					=	(*(anychar_p))
					;


		        BOOST_SPIRIT_DEBUG_RULE(clause_set);
		        BOOST_SPIRIT_DEBUG_RULE(clause);
		        BOOST_SPIRIT_DEBUG_RULE(literal_vector);
		        BOOST_SPIRIT_DEBUG_RULE(term);
		        BOOST_SPIRIT_DEBUG_RULE(function);
		        BOOST_SPIRIT_DEBUG_RULE(variable);
		        BOOST_SPIRIT_DEBUG_RULE(constant);
		        BOOST_SPIRIT_DEBUG_RULE(chr);
		        BOOST_SPIRIT_DEBUG_RULE(cnst);
		        BOOST_SPIRIT_DEBUG_RULE(vrb);
		        BOOST_SPIRIT_DEBUG_RULE(wrd);
		        BOOST_SPIRIT_DEBUG_RULE(eql);
		        BOOST_SPIRIT_DEBUG_RULE(term_list);
		        BOOST_SPIRIT_DEBUG_RULE(comment);
	        }

	        rule<ScannerT> clause_set, clause, literal_vector, term, function, variable, constant, chr, eql, wrd, cnst, vrb, term_list, comment;

	        rule<ScannerT> const&
	        start() const { return clause_set; }

	    };
	};

namespace Private
{
	struct InsertClause
	{
	public:
		InsertClause(Result &result) : m_result( result ) {}
		template <typename IteratorT>
		void operator()( IteratorT, IteratorT ) const
		{
			m_result.insertClause();
		}
	private:
		Result &m_result;
	};

	struct CreateClause
	{
	public:
		CreateClause(Result &result) : m_result( result ) {}
		template <typename IteratorT>
		void operator()( IteratorT, IteratorT ) const
		{
			m_result.createClause();
		}
	private:
		Result &m_result;
	};

	struct InsertLiteral
	{
	public:
		InsertLiteral(Result &result, bool pos) : m_result( result ), positive(pos) {}
		template <typename IteratorT>
		void operator()( IteratorT, IteratorT ) const
		{
			m_result.insertLiteral(positive);
		}
	private:
		Result &m_result;
		bool positive;
	};

	struct CreateFunctionName
	{
	public:
		CreateFunctionName(Result &result) : m_result( result ) {}
		template <typename IteratorT>
		void operator()(IteratorT first, IteratorT last) const
		{
			std::string str(first, last);
			m_result.createFunctionName(str);
		}
	private:
		Result &m_result;
	};

	struct CreateFunction
	{
	public:
		CreateFunction(Result &result) : m_result( result ) {}
		template <typename IteratorT>
		void operator()( IteratorT, IteratorT ) const
		{
			m_result.createFunction();
		}
	private:
		Result &m_result;
	};

	struct CreateVariable
	{
	public:
		CreateVariable(Result &result) : m_result( result ) {}
		template <typename IteratorT>
		void operator()(IteratorT first, IteratorT last) const
		{
			std::string str(first, last);
			m_result.createVariable(str);
		}
		template <typename IteratorT>
		void operator()(IteratorT c) const
		{
			std::string str(1, c);
			m_result.createVariable(str);
		}
	private:
		Result &m_result;
	};

	struct CreateConstant
	{
	public:
		CreateConstant(Result &result) : m_result( result ) {}
		template <typename IteratorT>
		void operator()(IteratorT first, IteratorT last) const
		{
			std::string str(first, last);
			m_result.createConstant(str);
		}
		template <typename IteratorT>
		void operator()(IteratorT c) const
		{
			std::string str(1, c);
			m_result.createConstant(str);
		}
	private:
		Result &m_result;
	};

	struct AddTerm
	{
	public:
		AddTerm(Result &result) : m_result( result ) {}
		template <typename IteratorT>
		void operator()( IteratorT, IteratorT ) const
		{
			m_result.addTerm();
		}
	private:
		Result &m_result;
	};

}

#endif /* PROVER9GRAMMER_H_ */
