#include "ResolvedLiteralFactorizer.h"

using namespace std;
using namespace NS_PROOFLIB;

ResolvedLiteralFactorizer::ResolvedLiteralFactorizer()
{
}

ResolvedLiteralFactorizer::~ResolvedLiteralFactorizer()
{
}

set<pair<Clause*, unsigned int>, ClausePairCmp> ResolvedLiteralFactorizer::getFactorsSet(Clause *c, ClauseContext *cc, unsigned int ind, bool indPos)
{
	return getRecFactorsSet(c, cc, c, ind, indPos);
}

set<pair<Clause*, unsigned int>, ClausePairCmp> ResolvedLiteralFactorizer::getRecFactorsSet(Clause *c, ClauseContext *cc, Clause *grandparent, unsigned int ind, bool indPos)
{
	set<pair<Clause*, unsigned int>, ClausePairCmp> ret;
	Term *t1;
	Term *t2;
	Unifier *unifier;
	unsigned int j;

	typedef std::vector<Term*>::const_iterator LI;

	ret.insert(pair<Clause*, int>(c,ind));

	if (indPos)
	{
		// positive literals
		vector<Term*> v = c->getPositiveLiterals();
		t1 = v[ind];
		for (j = 0; j<v.size(); j++)
		{
			if (j != (ind))
			{
				t2 = v[j];
				unifier = new UnifierImp();
				unifier->initialize(cc);
				unifier->unify(t1, t2);
				if (unifier->getStatus() == Unifier::OK)
				{
					vector<Term*> positive;
					// remove the ith element from the positive literals of the new clause
					for (unsigned int k = 0; k<v.size(); k++)
					{
						if (k != j)
						{
							positive.push_back(v[k]);
						}
					}
					unsigned int newInd = ind;
					// if the removed literal has smaller index than the compared to (pivot)
					// index then the pivot index must be reduced by one
					if (j < ind)
					{
						newInd--;
					}
					Clause *factor = new Factor(indPos, positive, c->getNegativeLiterals(), c->getVariableNameGenerator());
					factor->setClauseContext(unifier);
					unifier->addParent(grandparent);
					factor->addParent(unifier);
					// recursive call to add all factors (including the current)
					set<pair<Clause*, unsigned int>, ClausePairCmp> tmpSet = getRecFactorsSet(factor, unifier, grandparent, newInd, indPos);
					ret.insert(tmpSet.begin(), tmpSet.end());
				}
			}
		}
	}
	else
	{
		// negative literals
		vector<Term*> v = c->getNegativeLiterals();
		t1 = v[ind];
		for (j = 0; j<v.size(); j++)
		{
			if (j != (ind))
			{
				t2 = v[j];
				unifier = new UnifierImp();
				unifier->initialize(cc);
				unifier->unify(t1, t2);
				if (unifier->getStatus() == Unifier::OK)
				{
					vector<Term*> negative;
					// remove the ith element from the positive literals of the new clause
					for (unsigned int k = 0; k<v.size(); k++)
					{
						if (k != j)
						{
							negative.push_back(v[k]);
						}
					}
					unsigned int newInd = ind;
					// if the removed literal has smaller index than the compared to (pivot)
					// index then the pivot index must be reduced by one
					if (j < ind)
					{
						newInd--;
					}
					Clause *factor = new Factor(indPos, c->getPositiveLiterals(), negative, c->getVariableNameGenerator());
					factor->setClauseContext(unifier);
					unifier->addParent(grandparent);
					factor->addParent(unifier);
					// recursive call to add all factors (including the current)
					set<pair<Clause*, unsigned int>, ClausePairCmp> tmpSet = getRecFactorsSet(factor, unifier, grandparent, newInd, indPos);
					ret.insert(tmpSet.begin(), tmpSet.end());
				}
			}
		}
	}

	// all factors' unifiers are children of the initiating clause
	return ret;
}
