#include "SimpleParaModulator.h"

using namespace std;
using namespace NS_PROOFLIB;

SimpleParaModulator::SimpleParaModulator()
{
}

SimpleParaModulator::~SimpleParaModulator()
{
}

// does the first modulation it finds and returns the new element.
// must try to unify each subterm of t2 with both terms in t1 (i.e. with both t and s in s = t)
// Term must be = of two other terms
set<ParaModulant*> SimpleParaModulator::modulate(Clause *c1, Clause *c2, int index1, bool isPos1, int index2, bool isPos2)
{
	Term *t1;
	if (isPos1)
	{
		t1 = c1->getPositiveLiterals().at(index1);
	}
	else
	{
		t1 = c1->getNegativeLiterals().at(index1);
	}
	
	Term *t2;
	if (isPos2)
	{
		t2 = c2->getPositiveLiterals().at(index2);
	}
	else
	{
		t2 = c2->getNegativeLiterals().at(index2);
	}
	
	// extract s1 and s2 (t1 ^= s1 = s2)
	if (t1->getSymbol() != "=")
	{
		throw new ResolvingException("First term must be equality in paramodulation");
	}
	if (typeid(*t1) != typeid(Function))
	{
		throw new ResolvingException("First term must be equality predicate");
	}
	Function *f = dynamic_cast<Function *>(t1);
	if (f->getArity() != 2)
	{
		throw new ResolvingException("Equality predicate must have an arity of 2");
	}
	Term *s1 = f->getSub(1);
	Term *s2 = f->getSub(2);
	
	// try to unify s1 with a subterm of t2
	pair<Unifier*, Term*> p = extractModulant(s1, s2, t2, c1->getClauseContext(), c2->getClauseContext(), c1->getVariableNameGenerator()->pdb);
	
	// if no unifer of a subterm of t2 was found with s1 try to do it with s2
	if (p.first == NULL || p.first->getStatus() != Unifier::OK)
	{
		p = extractModulant(s2, s1, t2, c1->getClauseContext(), c2->getClauseContext(), c1->getVariableNameGenerator()->pdb);
	}
	
	ParaModulant *ret = NULL;
	// use the resulted term to create the new para modulant
	if (p.first != NULL && p.first->getStatus() == Unifier::OK)
	{
		ret = buildModulant(c1, c2, p.first, p.second, index1, isPos1, index2, isPos2);
	}
	set<ParaModulant*> st;
	st.insert(ret);
	return st;
}

// return after the first subterm is found
pair<Unifier*, Term*> SimpleParaModulator::extractModulant(Term *s1, Term *s2, Term *t2, ClauseContext *cc1, ClauseContext *cc2, ProofDatabase *pdb)
{
	return recExtractModulant(s1, s2, t2, cc1, cc2, pdb);
}

pair<Unifier*, Term*> SimpleParaModulator::recExtractModulant(Term *s1, Term *s2, Term *t2, ClauseContext *cc1, ClauseContext *cc2, ProofDatabase *pdb)
{
	// if t2 is variable then do not pmodulate
	if (typeid(*t2) == typeid(Variable))
	{
		return pair<Unifier*, Term*>(NULL,NULL);
	}
	else
	{
		// try to unify t2 and s1
		Unifier *unifier = unifAlg->unify(s1, t2, cc1, cc2);
		if (unifier->getStatus() == Unifier::OK)
    	{
			// return s2 instead of t2
			return pair<Unifier*, Term*>(unifier, s2);
    	}
		else
		{
			// if t2 is a function try to call recursively on all subterms of f
			if (typeid(*t2) == typeid(Function))
			{
				Function *f2 = dynamic_cast<Function *>(t2);		
				for (unsigned int i=1; i<=f2->getArity(); i++)
				{
					pair<Unifier*, Term*> p = recExtractModulant(s1, s2, f2->getSub(i), cc1, cc2, pdb);
					if (p.first != NULL)
					{
						// managed to find a subterm which unify, recuresively build the replacement
						TermList l;
						for (unsigned int j=1; j<=f2->getArity(); j++)
						{
							if (j != i)
							{
								l.addRight(f2->getSub(j));
							}
							else
							{
								// replace the term with the recuresively generated one
								l.addRight(p.second);
							}
						}
						Function *ret = pdb->newFunction(f2->getSymbol().c_str(), l);
						return pair<Unifier*, Term*>(unifier, ret);
					}
				}
			}
			else
			{
				// constant which does not unify, fail
				return pair<Unifier*, Term*>(NULL,NULL);
			}
		}
	}
	
	// fail
	return pair<Unifier*, Term*>(NULL,NULL);
}

ParaModulant *SimpleParaModulator::buildModulant(Clause *c1, Clause *c2, Unifier *unifier, NS_PROOFLIB::Term *t, int index1, bool isPos1, int index2, bool isPos2)
{
	vector<Term*> pos;
	vector<Term*> neg;

	typedef std::vector<Term*>::const_iterator LI;

	bool isPos = true;
	int counter = 0;
	for (LI it = c1->getPositiveLiterals().begin(); it != c1->getPositiveLiterals().end(); ++it)
	{
		if (!isPos1 || counter != index1)
		{
			pos.push_back(*it);
		}
		counter++;
	}

	counter = 0;
	for (LI it = c1->getNegativeLiterals().begin(); it != c1->getNegativeLiterals().end(); ++it)
	{
		if (isPos1 || counter != index1)
		{
			neg.push_back(*it);
		}
		counter++;
	}

	counter = 0;
	for (LI it = c2->getPositiveLiterals().begin(); it != c2->getPositiveLiterals().end(); ++it)
	{
		// insert term if is not positive or if is positive but not the given index
		// insert t otherwise
		if (!isPos2 || counter != index2)
		{
			pos.push_back(*it);
		}
		else
		{
			pos.push_back(t);
		}
		counter++;
	}

	counter = 0;
	for (LI it = c2->getNegativeLiterals().begin(); it != c2->getNegativeLiterals().end(); ++it)
	{
		if (isPos2 || counter != index2)
		{
			neg.push_back(*it);
		}
		else
		{
			isPos = false;
			neg.push_back(t);
		}
		counter++;
	}
	
	ParaModulant *res = new ParaModulant(pos, neg, Resolvent::NORMAL, c1->getVariableNameGenerator(), isPos);

	unifier->addParent(c1);
	unifier->addParent(c2);

	res->setClauseContext(unifier);
	
	return res;
}