#include "UnificationResolver.h"
#include <iostream>

using namespace NS_PROOFLIB;
using namespace std;

UnificationResolver::UnificationResolver(UnificationProofEnvironmentExtension *e)
{
	env = e;
}

UnificationResolver::~UnificationResolver()
{
}

/**
    Call unifier interface or paramodulator
*/
set<Resolvent*, ClausePointerCmp> UnificationResolver::resolve(Clause *c1, Clause *c2, ClauseStrategyRuling *s)
{
    // match all terms and call unification resolver which will doo unification or paramodulation (or nothing)
	/*  need to add all the following resolvents:
        1) paramodulants on t2 using t1 if it is = (return all options)
        2) paramodulants on t1 using t2 if it is = (return all options)
        3) resolvents if t1 and t2 are of different polarity
    */
	typedef vector<Term *>::const_iterator VIT;

	int indL1 = s->getFirstClauseLiteralIndex();
	int indL2 = s->getSecondClauseLiteralIndex();

	Term *t1;
	Term *t2;

	vector<Term*> pos1 = c1->getPositiveLiterals();
	vector<Term*> pos2 = c2->getPositiveLiterals();

	vector<Term*> neg1 = c1->getNegativeLiterals();
	vector<Term*> neg2 = c2->getNegativeLiterals();

    if (s->isFirstPositive())
    {
        t1 = pos1.at(indL1);
    }
    else
    {
        t1 = neg1.at(indL1);
    }

    if (s->isSecondPositive())
    {
        t2 = pos2.at(indL2);
    }
    else
    {
        t2 = neg2.at(indL2);
    }

    set<Resolvent*, ClausePointerCmp> ret;
	Resolvent *res;

    bool toUnify = true;

    if (t1->getSymbol() == "=" && s->isFirstPositive())
    {
        set<ParaModulant*> tmp = env->getParaModulator()->modulate(c1, c2, s->getFirstClauseLiteralIndex(), s->isFirstPositive(), s->getSecondClauseLiteralIndex(), s->isSecondPositive());
        // para modulants cannot be of type empty therefore they are normal (cannot be illegal)
        ret.insert(tmp.begin(), tmp.end());
    }

    if (t2->getSymbol() == "="  && s->isSecondPositive())
    {
        set<ParaModulant*> tmp = env->getParaModulator()->modulate(c2, c1, s->getSecondClauseLiteralIndex(), s->isSecondPositive(), s->getFirstClauseLiteralIndex(), s->isFirstPositive());
        ret.insert(tmp.begin(), tmp.end());
    }

    if (toUnify && s->isFirstPositive() != s->isSecondPositive())
    {
        Unifier *unifier = env->getUnificationAlgorithm()->getUnifiers(c1, c2, s->getFirstClauseLiteralIndex(), s->isFirstPositive(), s->getSecondClauseLiteralIndex(), s->isSecondPositive());
        if (unifier->getStatus() == Unifier::OK)
    	{
    		/*
    		 * try to factorize each clause with the resolved upon literal
    		 */
    		set<pair<Clause*, unsigned int>, ClausePairCmp> factors1 = env->getFactorizer()->getFactorsSet(c1, unifier, s->getFirstClauseLiteralIndex(), s->isFirstPositive());
    		set<pair<Clause*, unsigned int>, ClausePairCmp> factors2 = env->getFactorizer()->getFactorsSet(c2, unifier, s->getSecondClauseLiteralIndex(), s->isSecondPositive());

    		typedef set<pair<Clause*, unsigned int>, ClausePairCmp>::const_iterator LCI;

    		Clause *fac1;
    		Clause *fac2;
    		unsigned int index1;
    		unsigned int index2;
    		Resolvent *res;
    		// try all the combinations of all factors
    		for (LCI it1 = factors1.begin(); it1 != factors1.end(); ++it1)
    		{
    			fac1 = (*it1).first;
    			index1 = (*it1).second;
    			for (LCI it2 = factors2.begin(); it2 != factors2.end(); ++it2)
    			{
    				fac2 = (*it2).first;
    				index2 = (*it2).second;

    				vector<Term*> poss1 = fac1->getPositiveLiterals();
    				vector<Term*> poss2 = fac2->getPositiveLiterals();

    				vector<Term*> negg1 = fac1->getNegativeLiterals();
    				vector<Term*> negg2 = fac2->getNegativeLiterals();

    				pair<vector<Term*>, vector<Term*> > p = buildLiterals(s->isFirstPositive(), index1, index2, poss1, negg1, poss2, negg2);
    				vector<Term*> pos = p.first;
    				vector<Term*> neg = p.second;
    				if (pos.empty() && neg.empty())
    				{
    					res = new Resolvent(pos, neg, Resolvent::EMPTY, c1->getVariableNameGenerator());
    				}
    				else
    				{
    					res = new Resolvent(pos, neg, Resolvent::NORMAL, c1->getVariableNameGenerator());
    				}
    				Unifier *newc = new UnifierImp(*unifier);
    				newc->merge(c1->getClauseContext());
    				newc->merge(c2->getClauseContext());
    				//newc->addParent(s);
    				newc->addParent(fac1);
    				newc->addParent(fac2);

    				res->setClauseContext(newc);
    				//res->addParent(fac1);
    				//res->addParent(fac2);
    				ret.insert(res);
    			}
    		}
    	}
    }

	return ret;
}

pair<vector<Term*>, vector<Term*> > UnificationResolver::buildLiterals(bool firstIsPositive, int indL1, int indL2, vector<Term*> &pos1, vector<Term*> &neg1, vector<Term*> &pos2, vector<Term*> &neg2)
{
	vector<Term*> pos;
	vector<Term*> neg;

	typedef std::vector<Term*>::const_iterator LI;

	bool secondIsPositive = !firstIsPositive;

	int counter = 0;
	for (LI it = pos1.begin(); it != pos1.end(); ++it)
	{
		if (!firstIsPositive || counter != indL1)
		{
			pos.push_back(*it);
		}
		counter++;
	}

	counter = 0;
	for (LI it = neg1.begin(); it != neg1.end(); ++it)
	{
		if (firstIsPositive || counter != indL1)
		{
			neg.push_back(*it);
		}
		counter++;
	}

	counter = 0;
	for (LI it = pos2.begin(); it != pos2.end(); ++it)
	{
		if (!secondIsPositive || counter != indL2)
		{
			pos.push_back(*it);
		}
		counter++;
	}

	counter = 0;
	for (LI it = neg2.begin(); it != neg2.end(); ++it)
	{
		if (secondIsPositive || counter != indL2)
		{
			neg.push_back(*it);
		}
		counter++;
	}

	return pair<vector<Term*>, vector<Term*> >(pos, neg);
}
