#include "UnifierImp.h"

#include <typeinfo>
#include <iostream>

using namespace NS_PROOFLIB;
using namespace std;

UnifierImp::UnifierImp()
{
	setStatus(OK);
}

UnifierImp::UnifierImp(Unifier &u)
:Unifier(u)
{
	
}

UnifierImp::~UnifierImp()
{
}

void UnifierImp::unify(Term *tt1, Term *tt2)
{
	Term *t1 = getVariableInContext(tt1);
	Term *t2 = getVariableInContext(tt2);
	
	// t1 is a variable
	if (typeid(*t1) == typeid(Variable))
	{
		Variable *v = dynamic_cast<Variable*>(t1);
		// and occurs in t2
		if (isContainedIn(v, t2))
		{
			setStatus(OccurCheck);
		}
		// does not occur in t2
		else
		{
			mapVariableToTerm(v, t2);
			//setStatus(OK);
		}
	}
	// t2 is a variable
	else if (typeid(*t2) == typeid(Variable))
	{
		Variable *v = dynamic_cast<Variable*>(t2);
		// and occurs in t1
		if (isContainedIn(v, t1))
		{
			setStatus(OccurCheck);
		}
		// does not occur in t1
		else
		{
			mapVariableToTerm(v, t1);
			//setStatus(OK);
		}
	}
	// t1 or t2 is a constant
	else if (typeid(*t1) == typeid(Constant) || typeid(*t2) == typeid(Constant))
	{
		// and have the same symbol
		if (*t1 == *t2)
		{
			//setStatus(OK);
		}
		// do not have the same symbol
		else
		{
			setStatus(SymbolClash);
		}
	}
	// both are functions
	else
	{	
		// have the same symbol
		if (t1->getSymbol() == t2->getSymbol())
		{
			Function *f1 = dynamic_cast<Function *>(t1);
			Function *f2 = dynamic_cast<Function *>(t2);
			
			// maybe should check he arity is also the same but should be taken care by user
			for (unsigned int i=1; i<=f1->getArity(); i++)
			{
				if (getStatus() == OK)
				{
					unify(f1->getSub(i), f2->getSub(i));
				}
				else
				{
					break;
				}
			}
		}
		else
		{
			setStatus(SymbolClash);
		}
	}
}

bool UnifierImp::isContainedIn(Variable *t1, Term *told)
{
	// try to map it from context first
	Term *t2 = getVariableInContext(told);
	
	// if t2 is a variable
	if (typeid(*t2) == typeid(Variable))
	{	
		return t1->getSymbol() == t2->getSymbol(); 
	}
	// if t2 is a constant
	else if (typeid(*t2) == typeid(Constant))
	{
		return false;
	}
	// t2 is a function
	else
	{
		Function *f2 = dynamic_cast<Function *>(t2);
		
		for (unsigned int i=1; i<=f2->getArity(); i++)
		{
			if (isContainedIn(t1, f2->getSub(i)))
			{
				return true;
			}
		}
		return false;
	}
}