#include "ClauseFeatureVector.h"

using namespace std;
using namespace NS_PROOFLIB;

ClauseFeatureVector::ClauseFeatureVector(int s)
{
	size = s;
	depth = new unsigned int[size*2];
	for (int i = 0; i<size*2; i++)
	{
		depth[i] = 0;
	}
}

ClauseFeatureVector::~ClauseFeatureVector()
{
	delete[] depth;
}

void ClauseFeatureVector::init(Clause* c, map<string, unsigned int> *functionIndex)
{
	// update recursively the values for the depth of the different symbols
	typedef std::vector<Term*>::const_iterator LI;
	for (LI it = c->getPositiveLiterals().begin(); it != c->getPositiveLiterals().end(); ++it)
	{
		recUpdate(*it, 1, 0, c->getClauseContext(), functionIndex);
	}
	for (LI it = c->getNegativeLiterals().begin(); it != c->getNegativeLiterals().end(); ++it)
	{
		recUpdate(*it, 1, 1, c->getClauseContext(), functionIndex);
	}
}

void ClauseFeatureVector::recUpdate(Term* t, unsigned int curDepth, int negCoeff, ClauseContext *c, map<string, unsigned int> *functionIndex)
{
	if (typeid(*t) == typeid(Variable))
	{
		t = c->getVariableInContext(t);
	}

	if (typeid(*t) == typeid(Constant))
	{
		int loc = (*functionIndex)[t->getSymbol()]*2+negCoeff;
		if (depth[loc] < curDepth)
		{
			depth[loc] = curDepth;
		}
	}
	else if (typeid(*t) == typeid(Function))
	{
		Function *f = dynamic_cast<Function *>(t);
		int loc = (*functionIndex)[t->getSymbol()]*2+negCoeff;
		if (depth[loc] < curDepth)
		{
			depth[loc] = curDepth;
		}			
		for (unsigned int i=1; i<=f->getArity(); i++)
		{
			recUpdate(f->getSub(i), curDepth+1, negCoeff, c, functionIndex);
		}
	}
}

unsigned int ClauseFeatureVector::getDepthAtIndex(unsigned int index)
{
	return depth[index];
}