#ifndef CLAUSEFEATUREVECTORINDEX_H_
#define CLAUSEFEATUREVECTORINDEX_H_

#include "Trie.h"
#include "Clause.h"
#include "ClauseFeatureVector.h"
#include <set>
#include <string>
#include <map>
#include <stack>
#include <vector>
#include "Term.h"

class SubsumersIterator : public TrieIterator
{
	ClauseFeatureVector *vector;
	std::stack<std::pair<Trie*,unsigned int> > dfs;
	std::stack<Clause* > clauses;
public:
	SubsumersIterator(ClauseFeatureVector *cfv, Trie*);
	~SubsumersIterator();
	Clause *next();
};

/*
 * This algorithm is based on Stephan Schulz "Simple and Efficient Clause Subsumption
 * with Feature Vector Indexing". 
 * The features are the depth of function symbols in the clause so
 * the feature vector for p(f(X)) :- q(Y) given that the fnction index is p+ = 0, p- = 1, 
 * f+ = 2, f- = 3, q+ = 4, q- = 5 is <1 0 2 0 0 1>. We test for subsuming only clauses with
 * smaller or equal vector indices.
 */
class ClauseFeatureVectorIndex
{
	Trie tree; // represents symbols depth function
	// each symbol is mapped to an index
	std::map<std::string, unsigned int> *functionIndex; // (2 * i for positive, 2 * i + 1 for negative)
public:
	ClauseFeatureVectorIndex();
	virtual ~ClauseFeatureVectorIndex();
	// initialize the functionIndex
	void init(std::set<Clause*>*s);
	void insert(Clause*);
	// do not forget to clean the iterator after usage
	TrieIterator *getPossibleSubsumers(Clause *c);
	std::string toString();
private:
	void addClauseToIndex(ClauseFeatureVector &v, Clause *c);
	void addSymbols(std::set<std::string> &symbols, NS_PROOFLIB::Term* c, ClauseContext *c);
};

#endif /*CLAUSEFEATUREVECTORINDEX_H_*/
