#include "ForwardSubsumptionPostProcessor.h"

using namespace std;

ForwardSubsumptionPostProcessor::ForwardSubsumptionPostProcessor()
{
}

ForwardSubsumptionPostProcessor::~ForwardSubsumptionPostProcessor()
{
    delete alg;
}

void ForwardSubsumptionPostProcessor::setSubsumptionAlgorithm(SubsumptionAlgorithm *a)
{
	alg = a;
}

void ForwardSubsumptionPostProcessor::init(InputSet *is)
{
	set<Clause *> *st = is->getClauseSet();
	index.init(st);
	typedef set<Clause *>::const_iterator SC;
	for (SC it = st->begin(); it != st->end(); ++it)
	{
		index.insert(*it);
	}
}

bool ForwardSubsumptionPostProcessor::postProcess(Resolvent *r)
{
	TrieIterator *iterator = index.getPossibleSubsumers(r);
	Clause *var = NULL;
	bool ret = true;
	Clause *c;
	try
	{
		c = iterator->next();
		while (c != NULL)
		{
			if (var == NULL)
			{
				var = r->getVariant();
			}
			// check if r is subsumed by c
			if (isSubsumed(var,c))
			{
				ret = false;
				break;
			}
			
			c = iterator->next();
		}
	}
	catch (...)
	{
		delete var;
		delete iterator;
		throw;
	}
	if (!ret)
	{
		// clause is rejected
		string msg;
		msg.append(r->toString());
		msg.append(" is rejected as it is subsumed by ");
		msg.append(c->toString());
		msg.append("\n");
		fireEventPerformed(InputSetEvent(InputSetEvent::CLAUSE_REFUSED, r, msg));
	}
	return ret;
}

void ForwardSubsumptionPostProcessor::setEventPerformed(InputSetEvent ise)
{
	switch (ise.getType())
	{
	case InputSetEvent::CLAUSE_ADDED:
		index.insert(ise.getClause());
	    break;
	case InputSetEvent::CLAUSE_DELETED:
		throw new ATPException("deletion not implemented yet in forward subsumption");
	case InputSetEvent::CLAUSE_REFUSED:
		//nothing to do
		break;
	}
}

bool ForwardSubsumptionPostProcessor::isSubsumed(Clause *subsumed, Clause *subsumer)
{
	return alg->isSubsumed(subsumed, subsumer);
}