#include "StillmanSubsumptionAlgorithm.h"

using namespace std;
using namespace NS_PROOFLIB;

StillmanSubsumptionAlgorithm::StillmanSubsumptionAlgorithm(SubsumptionProofEnvironmentExtension *e) : SubsumptionAlgorithm(e)
{
}

StillmanSubsumptionAlgorithm::~StillmanSubsumptionAlgorithm()
{
}

bool StillmanSubsumptionAlgorithm::isSubsumed(Clause *subsumed, Clause *subsumer)
{
	Clause *ground = NULL;
	try
	{
		ground = subsumed->getGroundInstance();
		bool ret = ST0(subsumer, ground,1, 1, NULL);
		delete ground;
		return ret;
	}
	catch (...)
	{
		delete ground;
		throw;
	}
}

bool StillmanSubsumptionAlgorithm::ST0(Clause *c, Clause *d, unsigned int i, unsigned int j, Unifier *u)
{
	vector<Term*> dpos = d->getPositiveLiterals();
	vector<Term*> dneg = d->getNegativeLiterals();
	vector<Term*> cneg = c->getNegativeLiterals();
	vector<Term*> cpos = c->getPositiveLiterals();

	unsigned int dsize = dpos.size() + dneg.size();
	unsigned int csize = cpos.size() + cneg.size();

	// d is exhausted
	if (j > dsize)
	{
		return false;
	}
	else
	{
		Unifier *unif;
		// search for a matching literal of d
		while (j <= dsize)
		{
			// terms must be of same sign (pos/neg)
			if ( (j<=dpos.size() && i <= cpos.size()) || (j > dpos.size() && i > cpos.size()) )
			{
				Term *t1 = getTerm(cpos, cneg, i); // indices starts from 1 and not 0
				Term *t2 = getTerm(dpos, dneg, j); // indices starts from 1 and not 0

				// unifier was not generated yet
				// here we initialize the unifier from the two contexts
				// we must ground the context of the subsumed clause
				if (u == NULL)
				{
					unif = env->getUnificationAlgorithmForSubsumption()->unify(t1,t2, c->getClauseContext(), d->getClauseContext());
				}
				else
				{
					unif = env->getUnificationAlgorithmForSubsumption()->unify(t1, t2, u);
				}
				if (unif->getStatus() == Unifier::OK)
				{
					// continue with algorithm
					break;
				}
			}
			j++;
		}

		// no matchin literal was found
		if (j > dsize)
		{
			return false;
		}
		else
		{
			// here unif is OK
			if (i == csize || ST0(c,d,i+1,1,unif))
			{
				return true;
			}
			else
			{
				// backtracking (to the former unifier)
				return ST0(c,d,i,j+1,u);
			}
		}
	}
}

// i starts from 1 and not 0
Term *StillmanSubsumptionAlgorithm::getTerm(vector<Term*> &pos, vector<Term*> &neg, unsigned int i)
{
	// i is pos
	if (i <= pos.size())
	{
		return pos.at(i-1);
	}
	// i is neg
	else
	{
		return neg.at(i - pos.size() - 1);
	}
}
