#include "Trie.h"

using namespace std;

Trie *Trie::dummy = new Trie();

Trie::Trie()
{
	clauses = NULL;
	children = NULL;
}

Trie::Trie(Trie &t)
{
	clauses = t.clauses;
	children = t.children;
}

Trie::~Trie()
{
	if (children != NULL)
	{
		for (vector<Trie*>::const_iterator it = children->begin(); it != children->end(); ++it)
		{
			delete *it;
		}
		delete children;
	}
	delete clauses;
}

Trie *Trie::getOrCreate(unsigned int index)
{
	if(children == NULL)
	{
		children = new vector<Trie*>(1, dummy);
	}
	// indices of symbols starts at 0
	if (index+1 > children->size())
	{
		// dynamically enlarge the vector
		children->resize(index+1, dummy);
	}
	Trie *trie = children->at(index);
	if (trie ==  dummy)
	{
		trie = new Trie();
		(*children)[index] = trie;
	}
	return trie;
}

void Trie::insertClause(Clause* c)
{
	if (clauses == NULL)
	{
		clauses = new set<Clause*>();
	}
	clauses->insert(c);
}

string Trie::toString()
{
	string ret;
	if (clauses != NULL)
	{
		ret.append("{");
		for (set<Clause*>::const_iterator it = clauses->begin(); it != clauses->end(); ++it)
		{
			ret.append((*it)->toString());
			ret.append("; ");
		}
		ret.append("}\n");
	}
	else
	{
		int ind = 0;
		for (vector<Trie*>::const_iterator it = children->begin(); it != children->end(); ++it)
		{
			Trie *cur = *it;
			if (cur != dummy)
			{
				int i = ind;
				std::string s;
				std::stringstream out;
				out << i;
				ret.append(out.str());
				ret.append(" - ");
				ret.append(cur->toString());
			}
			ind++;
		}
	}
	return ret;
}