#ifndef TRIE_H_
#define TRIE_H_

#include <set>
#include <vector>
#include "Clause.h"

class TrieIterator
{
public:
	virtual ~TrieIterator() {}
	virtual Clause *next() = 0;
};

/*
 * Recursive data structure of fixed depth (determined by symbols) 
 * for storing the vector index
 * Trie is set<Clause*> | vector<Trie>
 */
class SubsumerIterator;
class Trie
{
	friend class SubsumersIterator;
	// vector maps detpth of symbol to a Trie (there may be empty indices in the vector)
	// the symbools are stored outside the Trie in the vctor index
	std::vector<Trie*> *children; // NULL for leaves
	std::set<Clause*> *clauses; // NULL for inner nodes
public:
	static Trie *dummy;
	Trie();
	Trie(Trie&);
	virtual ~Trie();
	Trie *getOrCreate(unsigned int index);
	void insertClause(Clause*);
	std::string toString();
};

#endif /*TRIE_H_*/
